@@ -1,4 +1,4 @@
-// Copyright © 2018–2022 Trevor Spiteri
+// Copyright © 2018–2023 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -14,7 +14,6 @@
 // <https://opensource.org/licenses/MIT>.
 
 use crate::{
-    int256::{self, I256, U256},
     traits::ToFixed,
     types::extra::{LeEqU128, LeEqU16, LeEqU32, LeEqU64, LeEqU8},
     FixedI128, FixedI16, FixedI32, FixedI64, FixedI8, FixedU128, FixedU16, FixedU32, FixedU64,
@@ -222,7 +221,7 @@ macro_rules! shift_all {
 
 macro_rules! fixed_arith {
     (
-        $Fixed:ident($Inner:ty, $LeEqU:ident, $bits_count:expr, $NonZeroInner:ident),
+        $Fixed:ident($Inner:ident, $LeEqU:ident, $bits_count:expr, $NonZeroInner:ident),
         $Signedness:tt
     ) => {
         if_signed! {
@@ -239,7 +238,8 @@ macro_rules! fixed_arith {
             type Output = $Fixed<Frac>;
             #[inline]
             fn mul(self, rhs: $Fixed<Frac>) -> $Fixed<Frac> {
-                let (ans, overflow) = overflowing_mul(self.to_bits(), rhs.to_bits(), Frac::U32);
+                let (ans, overflow) =
+                    $Inner::overflowing_mul(self.to_bits(), rhs.to_bits(), Frac::U32);
                 debug_assert!(!overflow, "overflow");
                 Self::from_bits(ans)
             }
@@ -250,7 +250,8 @@ macro_rules! fixed_arith {
         impl<Frac, RhsFrac: $LeEqU> MulAssign<$Fixed<RhsFrac>> for $Fixed<Frac> {
             #[inline]
             fn mul_assign(&mut self, rhs: $Fixed<RhsFrac>) {
-                let (ans, overflow) = overflowing_mul(self.to_bits(), rhs.to_bits(), RhsFrac::U32);
+                let (ans, overflow) =
+                    $Inner::overflowing_mul(self.to_bits(), rhs.to_bits(), RhsFrac::U32);
                 debug_assert!(!overflow, "overflow");
                 *self = Self::from_bits(ans);
             }
@@ -259,7 +260,8 @@ macro_rules! fixed_arith {
         impl<Frac, RhsFrac: $LeEqU> MulAssign<&$Fixed<RhsFrac>> for $Fixed<Frac> {
             #[inline]
             fn mul_assign(&mut self, rhs: &$Fixed<RhsFrac>) {
-                let (ans, overflow) = overflowing_mul(self.to_bits(), rhs.to_bits(), RhsFrac::U32);
+                let (ans, overflow) =
+                    $Inner::overflowing_mul(self.to_bits(), rhs.to_bits(), RhsFrac::U32);
                 debug_assert!(!overflow, "overflow");
                 *self = Self::from_bits(ans);
             }
@@ -269,7 +271,8 @@ macro_rules! fixed_arith {
             type Output = $Fixed<Frac>;
             #[inline]
             fn div(self, rhs: $Fixed<Frac>) -> $Fixed<Frac> {
-                let (ans, overflow) = overflowing_div(self.to_bits(), rhs.to_bits(), Frac::U32);
+                let (ans, overflow) =
+                    $Inner::overflowing_div(self.to_bits(), rhs.to_bits(), Frac::U32);
                 debug_assert!(!overflow, "overflow");
                 Self::from_bits(ans)
             }
@@ -586,63 +589,40 @@ fixed_arith! { FixedI32(i32, LeEqU32, 32, NonZeroI32), Signed }
 fixed_arith! { FixedI64(i64, LeEqU64, 64, NonZeroI64), Signed }
 fixed_arith! { FixedI128(i128, LeEqU128, 128, NonZeroI128), Signed }
 
-pub(crate) trait OverflowingMulDiv: Sized {
-    // 0 <= frac_nbits <= NBITS
-    fn overflowing_mul(self, rhs: Self, frac_nbits: u32) -> (Self, bool);
-    // -NBITS <= frac_nbits <= 2 * NBITS
-    fn overflowing_mul_add(self, mul: Self, add: Self, frac_nbits: i32) -> (Self, bool);
-    // 0 <= frac_nbits <= NBITS
-    fn overflowing_div(self, rhs: Self, frac_nbits: u32) -> (Self, bool);
-}
-
-#[inline]
-pub(crate) fn overflowing_mul<O: OverflowingMulDiv>(lhs: O, rhs: O, frac_nbits: u32) -> (O, bool) {
-    lhs.overflowing_mul(rhs, frac_nbits)
-}
-
-#[inline]
-pub(crate) fn overflowing_mul_add<O: OverflowingMulDiv>(
-    mul1: O,
-    mul2: O,
-    add: O,
-    frac_nbits: i32,
-) -> (O, bool) {
-    mul1.overflowing_mul_add(mul2, add, frac_nbits)
-}
-
-#[inline]
-pub(crate) fn overflowing_div<O: OverflowingMulDiv>(lhs: O, rhs: O, frac_nbits: u32) -> (O, bool) {
-    lhs.overflowing_div(rhs, frac_nbits)
-}
-
 macro_rules! mul_div_widen {
-    ($Single:ty, $Double:ty, $Signedness:tt, $Unsigned:ty) => {
-        impl OverflowingMulDiv for $Single {
+    ($Single:ident, $Double:ty, $Signedness:tt, $Unsigned:ty) => {
+        pub mod $Single {
+            // 0 <= frac_nbits <= NBITS
             #[inline]
-            fn overflowing_mul(self, rhs: $Single, frac_nbits: u32) -> ($Single, bool) {
+            pub const fn overflowing_mul(
+                lhs: $Single,
+                rhs: $Single,
+                frac_nbits: u32,
+            ) -> ($Single, bool) {
                 const NBITS: u32 = <$Single>::BITS;
                 let int_nbits: u32 = NBITS - frac_nbits;
-                let lhs2 = <$Double>::from(self);
-                let rhs2 = <$Double>::from(rhs) << int_nbits;
+                let lhs2 = lhs as $Double;
+                let rhs2 = (rhs as $Double) << int_nbits;
                 let (prod2, overflow) = lhs2.overflowing_mul(rhs2);
                 ((prod2 >> NBITS) as $Single, overflow)
             }
 
+            // -NBITS <= frac_nbits <= 2 * NBITS
             #[inline]
-            fn overflowing_mul_add(
-                self,
+            pub const fn overflowing_mul_add(
+                lhs: $Single,
                 mul: $Single,
                 add: $Single,
                 mut frac_nbits: i32,
             ) -> ($Single, bool) {
                 const NBITS: i32 = <$Single>::BITS as i32;
-                let self2 = <$Double>::from(self);
-                let mul2 = <$Double>::from(mul);
-                let prod2 = self2 * mul2;
+                let lhs2 = lhs as $Double;
+                let mul2 = mul as $Double;
+                let prod2 = lhs2 * mul2;
                 let (prod2, overflow2) = if frac_nbits < 0 {
                     frac_nbits += NBITS;
                     debug_assert!(frac_nbits >= 0);
-                    prod2.overflowing_mul(<$Double>::from(<$Unsigned>::MAX) + 1)
+                    prod2.overflowing_mul(<$Unsigned>::MAX as $Double + 1)
                 } else if frac_nbits > NBITS {
                     frac_nbits -= NBITS;
                     debug_assert!(frac_nbits <= NBITS);
@@ -671,12 +651,22 @@ macro_rules! mul_div_widen {
                 )
             }
 
+            // 0 <= frac_nbits <= NBITS
             #[inline]
-            fn overflowing_div(self, rhs: $Single, frac_nbits: u32) -> ($Single, bool) {
+            pub const fn overflowing_div(
+                lhs: $Single,
+                rhs: $Single,
+                frac_nbits: u32,
+            ) -> ($Single, bool) {
                 const NBITS: u32 = <$Single>::BITS;
-                let lhs2 = <$Double>::from(self) << frac_nbits;
-                let rhs2 = <$Double>::from(rhs);
-                let quot2 = lhs2 / rhs2;
+                let lhs2 = (lhs as $Double) << frac_nbits;
+                let rhs2 = (rhs as $Double);
+                // Overflow at this stage can only occur for signed when
+                // frac_nbits == NBITS, lhs == MIN, and rhs == -1.
+                // In that case, wrapped answer is $Double::MIN as $Single == 0
+                let (quot2, false) = lhs2.overflowing_div(rhs2) else {
+                    return (0, true);
+                };
                 let quot = quot2 as $Single;
                 let overflow = if_signed_unsigned!(
                     $Signedness,
@@ -698,21 +688,31 @@ mul_div_widen! { i16, i32, Signed, u16 }
 mul_div_widen! { i32, i64, Signed, u32 }
 mul_div_widen! { i64, i128, Signed, u64 }
 
-impl OverflowingMulDiv for u128 {
+pub mod u128 {
+    use crate::int256::{self, U256};
+    use core::num::NonZeroU128;
+
+    // 0 <= frac_nbits <= NBITS
     #[inline]
-    fn overflowing_mul(self, rhs: u128, frac_nbits: u32) -> (u128, bool) {
+    pub const fn overflowing_mul(lhs: u128, rhs: u128, frac_nbits: u32) -> (u128, bool) {
         if frac_nbits == 0 {
-            self.overflowing_mul(rhs)
+            lhs.overflowing_mul(rhs)
         } else {
-            let prod = int256::wide_mul_u128(self, rhs);
+            let prod = int256::wide_mul_u128(lhs, rhs);
             int256::overflowing_shl_u256_into_u128(prod, frac_nbits)
         }
     }
 
+    // -NBITS <= frac_nbits <= 2 * NBITS
     #[inline]
-    fn overflowing_mul_add(self, mul: u128, add: u128, mut frac_nbits: i32) -> (u128, bool) {
+    pub const fn overflowing_mul_add(
+        m1: u128,
+        m2: u128,
+        add: u128,
+        mut frac_nbits: i32,
+    ) -> (u128, bool) {
         // l * r + a
-        let mut prod = int256::wide_mul_u128(self, mul);
+        let mut prod = int256::wide_mul_u128(m1, m2);
 
         let mut overflow1 = false;
         if frac_nbits < 0 {
@@ -733,38 +733,60 @@ impl OverflowingMulDiv for u128 {
         (ans, overflow1 | overflow2 | overflow3)
     }
 
+    // 0 <= frac_nbits <= NBITS
     #[inline]
-    fn overflowing_div(self, rhs: u128, frac_nbits: u32) -> (u128, bool) {
+    pub const fn overflowing_div(lhs: u128, rhs: u128, frac_nbits: u32) -> (u128, bool) {
         if frac_nbits == 0 {
-            self.overflowing_div(rhs)
-        } else {
-            let lhs2 = U256 {
-                lo: self << frac_nbits,
-                hi: self >> (128 - frac_nbits),
-            };
-            let (quot2, _) = int256::div_rem_u256_u128(lhs2, rhs);
-            let quot = quot2.lo;
-            let overflow = quot2.hi != 0;
-            (quot, overflow)
+            return lhs.overflowing_div(rhs);
         }
+        let Some(rhs) = NonZeroU128::new(rhs) else {
+            panic!("division by zero");
+        };
+        overflowing_div_nz(lhs, rhs, frac_nbits)
+    }
+
+    // Neither rhs nor frac_nbits is zero.
+    const fn overflowing_div_nz(lhs: u128, rhs: NonZeroU128, frac_nbits: u32) -> (u128, bool) {
+        let lhs2 = if frac_nbits == 128 {
+            U256 { lo: 0, hi: lhs }
+        } else {
+            U256 {
+                lo: lhs << frac_nbits,
+                hi: lhs >> (128 - frac_nbits),
+            }
+        };
+        let (quot2, _) = int256::div_rem_u256_u128(lhs2, rhs);
+        let quot = quot2.lo;
+        let overflow = quot2.hi != 0;
+        (quot, overflow)
     }
 }
 
-impl OverflowingMulDiv for i128 {
+pub mod i128 {
+    use crate::int256::{self, I256};
+    use core::num::NonZeroI128;
+
+    // 0 <= frac_nbits <= NBITS
     #[inline]
-    fn overflowing_mul(self, rhs: i128, frac_nbits: u32) -> (i128, bool) {
+    pub const fn overflowing_mul(lhs: i128, rhs: i128, frac_nbits: u32) -> (i128, bool) {
         if frac_nbits == 0 {
-            self.overflowing_mul(rhs)
+            lhs.overflowing_mul(rhs)
         } else {
-            let prod = int256::wide_mul_i128(self, rhs);
+            let prod = int256::wide_mul_i128(lhs, rhs);
             int256::overflowing_shl_i256_into_i128(prod, frac_nbits)
         }
     }
 
+    // -NBITS <= frac_nbits <= 2 * NBITS
     #[inline]
-    fn overflowing_mul_add(self, mul: i128, add: i128, mut frac_nbits: i32) -> (i128, bool) {
+    pub const fn overflowing_mul_add(
+        m1: i128,
+        m2: i128,
+        add: i128,
+        mut frac_nbits: i32,
+    ) -> (i128, bool) {
         // l * r + a
-        let mut prod = int256::wide_mul_i128(self, mul);
+        let mut prod = int256::wide_mul_i128(m1, m2);
 
         let mut overflow1 = false;
         if frac_nbits < 0 {
@@ -793,20 +815,37 @@ impl OverflowingMulDiv for i128 {
         (ans, overflow1 | (shifted.hi != expected_hi))
     }
 
+    // 0 <= frac_nbits <= NBITS
     #[inline]
-    fn overflowing_div(self, rhs: i128, frac_nbits: u32) -> (i128, bool) {
+    pub const fn overflowing_div(lhs: i128, rhs: i128, frac_nbits: u32) -> (i128, bool) {
         if frac_nbits == 0 {
-            self.overflowing_div(rhs)
-        } else {
-            let lhs2 = I256 {
-                lo: (self << frac_nbits) as u128,
-                hi: self >> (128 - frac_nbits),
-            };
-            let (quot2, _) = int256::div_rem_i256_i128(lhs2, rhs);
-            let quot = quot2.lo as i128;
-            let overflow = quot2.hi != quot >> 127;
-            (quot, overflow)
+            return lhs.overflowing_div(rhs);
         }
+        let Some(rhs) = NonZeroI128::new(rhs) else {
+            panic!("division by zero");
+        };
+        overflowing_div_nz(lhs, rhs, frac_nbits)
+    }
+
+    // Neither rhs nor frac_nbits is zero.
+    const fn overflowing_div_nz(lhs: i128, rhs: NonZeroI128, frac_nbits: u32) -> (i128, bool) {
+        let lhs2 = if frac_nbits == 128 {
+            // In this case, overflow is possible in the wide division.
+            // Then, the wrapped answer is I256::MIN as i128 == 0
+            if lhs == i128::MIN && rhs.get() == -1 {
+                return (0, true);
+            }
+            I256 { lo: 0, hi: lhs }
+        } else {
+            I256 {
+                lo: (lhs << frac_nbits) as u128,
+                hi: lhs >> (128 - frac_nbits),
+            }
+        };
+        let (quot2, _) = int256::div_rem_i256_i128_no_overflow(lhs2, rhs);
+        let quot = quot2.lo as i128;
+        let overflow = quot2.hi != quot >> 127;
+        (quot, overflow)
     }
 }
 
@@ -996,7 +1035,7 @@ mod tests {
         match (I16F16::from_num(a).checked_rem_int(b), a.checked_rem(b)) {
             (Some(a), Some(b)) => assert_eq!(a, b),
             (None, None) => {}
-            (a, b) => panic!("mismatch {:?}, {:?}", a, b),
+            (a, b) => panic!("mismatch {a:?}, {b:?}"),
         }
         match (
             I16F16::from_num(a).checked_rem_euclid_int(b),
@@ -1004,7 +1043,7 @@ mod tests {
         ) {
             (Some(a), Some(b)) => assert_eq!(a, b),
             (None, None) => {}
-            (a, b) => panic!("mismatch {:?}, {:?}", a, b),
+            (a, b) => panic!("mismatch {a:?}, {b:?}"),
         }
     }
 
@@ -1116,9 +1155,8 @@ mod tests {
                 let a16 = U16F16::from_bits(a);
                 let a31 = U31F1::from_bits(a);
                 let a32 = U32F0::from_bits(a);
-                let nz = match NonZeroU32::new(b) {
-                    Some(s) => s,
-                    None => continue,
+                let Some(nz) = NonZeroU32::new(b) else {
+                    continue;
                 };
                 assert_eq!(a0 / nz, a0 / b);
                 assert_eq!(a0 % nz, a0 % b);
@@ -1158,9 +1196,8 @@ mod tests {
                 let a16 = I16F16::from_bits(a);
                 let a31 = I31F1::from_bits(a);
                 let a32 = I32F0::from_bits(a);
-                let nz = match NonZeroI32::new(b) {
-                    Some(s) => s,
-                    None => continue,
+                let Some(nz) = NonZeroI32::new(b) else {
+                    continue;
                 };
                 assert_eq!(a0 % nz, a0 % b);
                 assert_eq!(a1 % nz, a1 % b);
@@ -1231,56 +1268,56 @@ mod tests {
         let max = u64::MAX;
 
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max, nbits_2),
+            arith::u64::overflowing_mul_add(max, max, max, nbits_2),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max, nbits_2 - 1),
+            arith::u64::overflowing_mul_add(max, max, max, nbits_2 - 1),
             (0, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max - 1, nbits_2 - 1),
+            arith::u64::overflowing_mul_add(max, max, max - 1, nbits_2 - 1),
             (max, false)
         );
 
         let (min, max) = (i64::MIN, i64::MAX);
 
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max, nbits_2 - 2),
+            arith::i64::overflowing_mul_add(max, max, max, nbits_2 - 2),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max, nbits_2 - 3),
+            arith::i64::overflowing_mul_add(max, max, max, nbits_2 - 3),
             (min, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max - 1, nbits_2 - 3),
+            arith::i64::overflowing_mul_add(max, max, max - 1, nbits_2 - 3),
             (max, false)
         );
 
         assert_eq!(
-            arith::overflowing_mul_add(min, min, max, nbits_2 - 1),
+            arith::i64::overflowing_mul_add(min, min, max, nbits_2 - 1),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(min, min, max, nbits_2 - 2),
+            arith::i64::overflowing_mul_add(min, min, max, nbits_2 - 2),
             (min, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(min, min, max - 1, nbits_2 - 2),
+            arith::i64::overflowing_mul_add(min, min, max - 1, nbits_2 - 2),
             (max, false)
         );
 
         assert_eq!(
-            arith::overflowing_mul_add(max, min, -max, nbits_2 - 2),
+            arith::i64::overflowing_mul_add(max, min, -max, nbits_2 - 2),
             (min, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, min, -max, nbits_2 - 3),
+            arith::i64::overflowing_mul_add(max, min, -max, nbits_2 - 3),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, min, -max + 1, nbits_2 - 3),
+            arith::i64::overflowing_mul_add(max, min, -max + 1, nbits_2 - 3),
             (min, false)
         );
 
@@ -1289,56 +1326,56 @@ mod tests {
         let max = u128::MAX;
 
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max, nbits_2),
+            arith::u128::overflowing_mul_add(max, max, max, nbits_2),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max, nbits_2 - 1),
+            arith::u128::overflowing_mul_add(max, max, max, nbits_2 - 1),
             (0, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max - 1, nbits_2 - 1),
+            arith::u128::overflowing_mul_add(max, max, max - 1, nbits_2 - 1),
             (max, false)
         );
 
         let (min, max) = (i128::MIN, i128::MAX);
 
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max, nbits_2 - 2),
+            arith::i128::overflowing_mul_add(max, max, max, nbits_2 - 2),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max, nbits_2 - 3),
+            arith::i128::overflowing_mul_add(max, max, max, nbits_2 - 3),
             (min, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, max, max - 1, nbits_2 - 3),
+            arith::i128::overflowing_mul_add(max, max, max - 1, nbits_2 - 3),
             (max, false)
         );
 
         assert_eq!(
-            arith::overflowing_mul_add(min, min, max, nbits_2 - 1),
+            arith::i128::overflowing_mul_add(min, min, max, nbits_2 - 1),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(min, min, max, nbits_2 - 2),
+            arith::i128::overflowing_mul_add(min, min, max, nbits_2 - 2),
             (min, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(min, min, max - 1, nbits_2 - 2),
+            arith::i128::overflowing_mul_add(min, min, max - 1, nbits_2 - 2),
             (max, false)
         );
 
         assert_eq!(
-            arith::overflowing_mul_add(max, min, -max, nbits_2 - 2),
+            arith::i128::overflowing_mul_add(max, min, -max, nbits_2 - 2),
             (min, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, min, -max, nbits_2 - 3),
+            arith::i128::overflowing_mul_add(max, min, -max, nbits_2 - 3),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(max, min, -max + 1, nbits_2 - 3),
+            arith::i128::overflowing_mul_add(max, min, -max + 1, nbits_2 - 3),
             (min, false)
         );
     }
@@ -1350,68 +1387,68 @@ mod tests {
         let (zero, one, max) = (0u64, 1u64, u64::MAX);
 
         assert_eq!(
-            arith::overflowing_mul_add(zero, zero, max, -nbits),
+            arith::u64::overflowing_mul_add(zero, zero, max, -nbits),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, max, -nbits),
+            arith::u64::overflowing_mul_add(one, one, max, -nbits),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, zero, 1 - nbits),
+            arith::u64::overflowing_mul_add(one, one, zero, 1 - nbits),
             (max - max / 2, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, max, 1 - nbits),
+            arith::u64::overflowing_mul_add(one, one, max, 1 - nbits),
             (max / 2, true)
         );
 
         let (zero, one, min, max) = (0i64, 1i64, i64::MIN, i64::MAX);
 
         assert_eq!(
-            arith::overflowing_mul_add(zero, zero, max, -nbits),
+            arith::i64::overflowing_mul_add(zero, zero, max, -nbits),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, max, -nbits),
+            arith::i64::overflowing_mul_add(one, one, max, -nbits),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, -one, 1 - nbits),
+            arith::i64::overflowing_mul_add(one, one, -one, 1 - nbits),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, zero, 1 - nbits),
+            arith::i64::overflowing_mul_add(one, one, zero, 1 - nbits),
             (min, true)
         );
 
         assert_eq!(
-            arith::overflowing_mul_add(-one, -one, max, -nbits),
+            arith::i64::overflowing_mul_add(-one, -one, max, -nbits),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(-one, -one, -one, 1 - nbits),
+            arith::i64::overflowing_mul_add(-one, -one, -one, 1 - nbits),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(-one, -one, zero, 1 - nbits),
+            arith::i64::overflowing_mul_add(-one, -one, zero, 1 - nbits),
             (min, true)
         );
 
         assert_eq!(
-            arith::overflowing_mul_add(one, -one, max, -nbits),
+            arith::i64::overflowing_mul_add(one, -one, max, -nbits),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, -one, min, -nbits),
+            arith::i64::overflowing_mul_add(one, -one, min, -nbits),
             (min, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, -one, zero, 1 - nbits),
+            arith::i64::overflowing_mul_add(one, -one, zero, 1 - nbits),
             (min, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, -one, max, 1 - nbits),
+            arith::i64::overflowing_mul_add(one, -one, max, 1 - nbits),
             (-one, false)
         );
 
@@ -1420,68 +1457,68 @@ mod tests {
         let (zero, one, max) = (0u128, 1u128, u128::MAX);
 
         assert_eq!(
-            arith::overflowing_mul_add(zero, zero, max, -nbits),
+            arith::u128::overflowing_mul_add(zero, zero, max, -nbits),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, max, -nbits),
+            arith::u128::overflowing_mul_add(one, one, max, -nbits),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, zero, 1 - nbits),
+            arith::u128::overflowing_mul_add(one, one, zero, 1 - nbits),
             (max - max / 2, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, max, 1 - nbits),
+            arith::u128::overflowing_mul_add(one, one, max, 1 - nbits),
             (max / 2, true)
         );
 
         let (zero, one, min, max) = (0i128, 1i128, i128::MIN, i128::MAX);
 
         assert_eq!(
-            arith::overflowing_mul_add(zero, zero, max, -nbits),
+            arith::i128::overflowing_mul_add(zero, zero, max, -nbits),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, max, -nbits),
+            arith::i128::overflowing_mul_add(one, one, max, -nbits),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, -one, 1 - nbits),
+            arith::i128::overflowing_mul_add(one, one, -one, 1 - nbits),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, one, zero, 1 - nbits),
+            arith::i128::overflowing_mul_add(one, one, zero, 1 - nbits),
             (min, true)
         );
 
         assert_eq!(
-            arith::overflowing_mul_add(-one, -one, max, -nbits),
+            arith::i128::overflowing_mul_add(-one, -one, max, -nbits),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(-one, -one, -one, 1 - nbits),
+            arith::i128::overflowing_mul_add(-one, -one, -one, 1 - nbits),
             (max, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(-one, -one, zero, 1 - nbits),
+            arith::i128::overflowing_mul_add(-one, -one, zero, 1 - nbits),
             (min, true)
         );
 
         assert_eq!(
-            arith::overflowing_mul_add(one, -one, max, -nbits),
+            arith::i128::overflowing_mul_add(one, -one, max, -nbits),
             (max, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, -one, min, -nbits),
+            arith::i128::overflowing_mul_add(one, -one, min, -nbits),
             (min, true)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, -one, zero, 1 - nbits),
+            arith::i128::overflowing_mul_add(one, -one, zero, 1 - nbits),
             (min, false)
         );
         assert_eq!(
-            arith::overflowing_mul_add(one, -one, max, 1 - nbits),
+            arith::i128::overflowing_mul_add(one, -one, max, 1 - nbits),
             (-one, false)
         );
     }
@@ -1540,4 +1577,65 @@ mod tests {
         let x: Result<FixedI128<U125>, _> = "9.079999999999999999999".parse();
         assert!(x.is_err());
     }
+
+    #[test]
+    fn issue_51() {
+        use crate::types::*;
+        // these are for the bug:
+        assert_eq!(I0F8::MIN.overflowing_div(-I0F8::DELTA), (I0F8::ZERO, true));
+        assert_eq!(
+            I0F16::MIN.overflowing_div(-I0F16::DELTA),
+            (I0F16::ZERO, true)
+        );
+        assert_eq!(
+            I0F32::MIN.overflowing_div(-I0F32::DELTA),
+            (I0F32::ZERO, true)
+        );
+        assert_eq!(
+            I0F64::MIN.overflowing_div(-I0F64::DELTA),
+            (I0F64::ZERO, true)
+        );
+        assert_eq!(
+            I0F128::MIN.overflowing_div(-I0F128::DELTA),
+            (I0F128::ZERO, true)
+        );
+        // some extra tests:
+        assert_eq!(
+            I0F32::MIN.overflowing_div(I0F32::DELTA),
+            (I0F32::ZERO, true)
+        );
+        assert_eq!(I0F32::MIN.overflowing_div(I0F32::MIN), (I0F32::ZERO, true));
+        assert_eq!(
+            (-I0F32::MAX).overflowing_div(-I0F32::DELTA),
+            (I0F32::ZERO, true)
+        );
+        assert_eq!(
+            (-I0F32::MAX).overflowing_div(I0F32::DELTA),
+            (I0F32::ZERO, true)
+        );
+        assert_eq!(
+            (-I0F32::MAX).overflowing_div(I0F32::MIN),
+            (I0F32::from_bits(-2), true)
+        );
+        assert_eq!(
+            I0F128::MIN.overflowing_div(I0F128::DELTA),
+            (I0F128::ZERO, true)
+        );
+        assert_eq!(
+            I0F128::MIN.overflowing_div(I0F128::MIN),
+            (I0F128::ZERO, true)
+        );
+        assert_eq!(
+            (-I0F128::MAX).overflowing_div(-I0F128::DELTA),
+            (I0F128::ZERO, true)
+        );
+        assert_eq!(
+            (-I0F128::MAX).overflowing_div(I0F128::DELTA),
+            (I0F128::ZERO, true)
+        );
+        assert_eq!(
+            (-I0F128::MAX).overflowing_div(I0F128::MIN),
+            (I0F128::from_bits(-2), true)
+        );
+    }
 }
