@@ -1,4 +1,4 @@
-// Copyright © 2018–2022 Trevor Spiteri
+// Copyright © 2018–2023 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -13,10 +13,12 @@
 // <https://www.apache.org/licenses/LICENSE-2.0> and
 // <https://opensource.org/licenses/MIT>.
 
+#![allow(deprecated)]
+
 use crate::{
     types::extra::{LeEqU128, LeEqU16, LeEqU32, LeEqU64, LeEqU8},
     F128Bits, FixedI128, FixedI16, FixedI32, FixedI64, FixedI8, FixedU128, FixedU16, FixedU32,
-    FixedU64, FixedU8,
+    FixedU64, FixedU8, F128,
 };
 use az_crate::{Cast, CheckedCast, OverflowingCast, SaturatingCast, UnwrappedCast, WrappingCast};
 use half::{bf16, f16};
@@ -185,7 +187,7 @@ macro_rules! cast_num {
             $Fixed($LeEqU);
             i8, i16, i32, i64, i128, isize,
             u8, u16, u32, u64, u128, usize,
-            f16, bf16, f32, f64, F128Bits,
+            f16, bf16, f32, f64, F128, F128Bits,
         }
     )* };
 }
