@@ -1,4 +1,4 @@
-// Copyright © 2018–2022 Trevor Spiteri
+// Copyright © 2018–2023 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -13,7 +13,10 @@
 // <https://www.apache.org/licenses/LICENSE-2.0> and
 // <https://opensource.org/licenses/MIT>.
 
-use az_crate::WrappingAs;
+#[cfg(not(debug_assertions))]
+use core::hint;
+
+use core::num::{NonZeroI128, NonZeroU128};
 
 #[derive(Clone, Copy, Debug)]
 pub struct U256 {
@@ -28,60 +31,82 @@ pub struct I256 {
 }
 
 #[inline]
-pub fn u256_wrapping_as_i256(a: U256) -> I256 {
+pub const fn u256_wrapping_as_i256(a: U256) -> I256 {
     I256 {
         lo: a.lo,
-        hi: a.hi.wrapping_as::<i128>(),
+        hi: a.hi as i128,
     }
 }
 
 #[inline]
-pub fn wrapping_add_u256_u128(a: U256, b: u128) -> U256 {
+pub const fn wrapping_add_u256_u128(a: U256, b: u128) -> U256 {
     let (lo, carry) = a.lo.overflowing_add(b);
-    let hi = a.hi.wrapping_add(u128::from(carry));
+    let hi = a.hi.wrapping_add(carry as u128);
     U256 { lo, hi }
 }
 
 #[inline]
-pub fn overflowing_add_u128_u256(a: u128, b: U256) -> (u128, bool) {
+pub const fn overflowing_add_u128_u256(a: u128, b: U256) -> (u128, bool) {
     let (lo, carry) = a.overflowing_add(b.lo);
     (lo, carry | (b.hi != 0))
 }
 
 #[inline]
-pub fn overflowing_sub_u128_u256(a: u128, b: U256) -> (u128, bool) {
+pub const fn overflowing_sub_u128_u256(a: u128, b: U256) -> (u128, bool) {
     let (lo, borrow) = a.overflowing_sub(b.lo);
     (lo, borrow | (b.hi != 0))
 }
 
 #[inline]
-pub fn wrapping_neg_u256(a: U256) -> U256 {
+pub const fn wrapping_neg_u256(a: U256) -> U256 {
     let (lo, carry) = (!a.lo).overflowing_add(1);
-    let hi = (!a.hi).wrapping_add(u128::from(carry));
+    let hi = (!a.hi).wrapping_add(carry as u128);
     U256 { lo, hi }
 }
 
 #[inline]
-pub fn overflowing_add_i256_i128(a: I256, b: i128) -> (I256, bool) {
+pub const fn overflowing_add_i256_i128(a: I256, b: i128) -> (I256, bool) {
     let b = I256 {
-        lo: b.wrapping_as::<u128>(),
+        lo: b as u128,
         hi: b >> 127,
     };
     let (lo, carry) = a.lo.overflowing_add(b.lo);
     // b.hi is in {-1, 0}, and carry is in {0, 1}, so we can add them wrappingly
-    let b_hi_plus_carry = b.hi.wrapping_add(i128::from(carry));
+    let b_hi_plus_carry = b.hi.wrapping_add(carry as i128);
     let (hi, overflow) = a.hi.overflowing_add(b_hi_plus_carry);
     (I256 { lo, hi }, overflow)
 }
 
 #[inline]
-const fn u128_lo_hi(u: u128) -> (u128, u128) {
-    (u & !(!0 << 64), u >> 64)
+const fn u128_lo_hi(u: u128) -> (u64, u64) {
+    (u as u64, (u >> 64) as u64)
 }
 
 #[inline]
-const fn i128_lo_hi(i: i128) -> (i128, i128) {
-    (i & !(!0 << 64), i >> 64)
+const fn i128_lo_hi(i: i128) -> (u64, i64) {
+    (i as u64, (i >> 64) as i64)
+}
+
+#[inline]
+const fn mul_u64_u64(a: u64, b: u64) -> u128 {
+    (a as u128).wrapping_mul(b as u128)
+}
+
+#[inline]
+const fn mul_i64_i64(a: i64, b: i64) -> i128 {
+    (a as i128).wrapping_mul(b as i128)
+}
+
+#[inline]
+const fn mul_u64_i64(a: u64, b: i64) -> i128 {
+    let a = a as i64;
+    // if a has become negative, we need to add 2^64 * b to the answer
+    let correction = if a.is_negative() {
+        (b as i128) << 64
+    } else {
+        0
+    };
+    (a as i128).wrapping_mul(b as i128).wrapping_add(correction)
 }
 
 #[inline]
@@ -89,34 +114,38 @@ pub const fn wide_mul_u128(lhs: u128, rhs: u128) -> U256 {
     let (ll, lh) = u128_lo_hi(lhs);
     let (rl, rh) = u128_lo_hi(rhs);
     // 0 <= ll_rl <= 2^128 - 2^65 + 1; ll_rl unit is 1
-    let ll_rl = ll.wrapping_mul(rl);
+    let ll_rl = mul_u64_u64(ll, rl);
     // 0 <= lh_rl <= 2^128 - 2^65 + 1; lh_rl unit is 2^64
-    let lh_rl = lh.wrapping_mul(rl);
+    let lh_rl = mul_u64_u64(lh, rl);
     // 0 <= ll_rh <= 2^128 - 2^65 + 1; ll_rh unit is 2^64
-    let ll_rh = ll.wrapping_mul(rh);
+    let ll_rh = mul_u64_u64(ll, rh);
     // 0 <= lh_rh <= 2^128 - 2^65 + 1; lh_rh unit is 2^128
-    let lh_rh = lh.wrapping_mul(rh);
+    let lh_rh = mul_u64_u64(lh, rh);
 
     // 0 <= col0 <= 2^64 - 1
     // 0 <= col64a <= 2^64 - 2
     let (col0, col64a) = u128_lo_hi(ll_rl);
 
     // 0 <= col64b <= 2^128 - 2^64 - 1
-    let col64b = col64a.wrapping_add(lh_rl);
+    let col64b = (col64a as u128).wrapping_add(lh_rl);
 
-    // 0 <= col64c <= 2^128 - 1
-    // 0 <= col192 <= 1
-    let (col64c, col192) = col64b.overflowing_add(ll_rh);
-    let col192 = if col192 { 1u128 } else { 0u128 };
+    // 0 <= col64c <= 2^64 - 1
+    // 0 <= col128a <= 2^64 - 2
+    let (col64c, col128a) = u128_lo_hi(col64b);
+
+    // 0 <= col64d <= 2^128 - 2^64
+    let col64d = (col64c as u128).wrapping_add(ll_rh);
 
     // 0 <= col64 <= 2^64 - 1
-    // 0 <= col128 <= 2^64 - 1
-    let (col64, col128) = u128_lo_hi(col64c);
+    // 0 <= col128b <= 2^64 - 1
+    let (col64, col128b) = u128_lo_hi(col64d);
 
     // Since both col0 and col64 fit in 64 bits, ans0 sum will never overflow.
-    let ans0 = col0.wrapping_add(col64 << 64);
+    let ans0 = (col0 as u128) | ((col64 as u128) << 64);
     // Since lhs * rhs fits in 256 bits, ans128 sum will never overflow.
-    let ans128 = lh_rh.wrapping_add(col128).wrapping_add(col192 << 64);
+    let ans128 = lh_rh
+        .wrapping_add(col128a as u128)
+        .wrapping_add(col128b as u128);
     U256 {
         lo: ans0,
         hi: ans128,
@@ -128,13 +157,13 @@ pub const fn wide_mul_i128(lhs: i128, rhs: i128) -> I256 {
     let (ll, lh) = i128_lo_hi(lhs);
     let (rl, rh) = i128_lo_hi(rhs);
     // 0 <= ll_rl <= 2^128 - 2^65 + 1; ll_rl unit is 1; must be unsigned to hold all range!
-    let ll_rl = (ll as u128).wrapping_mul(rl as u128);
+    let ll_rl = mul_u64_u64(ll, rl);
     // -2^127 + 2^63 <= lh_rl <= 2^127 - 2^64 - 2^63 + 1; lh_rl unit is 2^64
-    let lh_rl = lh.wrapping_mul(rl);
+    let lh_rl = mul_u64_i64(rl, lh);
     // -2^127 + 2^63 <= ll_rh <= 2^127 - 2^64 - 2^63 + 1; ll_rh unit is 2^64
-    let ll_rh = ll.wrapping_mul(rh);
+    let ll_rh = mul_u64_i64(ll, rh);
     // -2^126 + 2^63 <= lh_rh <= 2^126; lh_rh unit is 2^128
-    let lh_rh = lh.wrapping_mul(rh);
+    let lh_rh = mul_i64_i64(lh, rh);
 
     // 0 <= col0 <= 2^64 - 1
     // 0 <= col64a <= 2^64 - 2
@@ -143,28 +172,23 @@ pub const fn wide_mul_i128(lhs: i128, rhs: i128) -> I256 {
     // -2^127 + 2^63 <= col64b <= 2^127 - 2^63 - 1
     let col64b = (col64a as i128).wrapping_add(lh_rl);
 
-    // -2^127 <= col64c <= 2^127 - 1
-    // -1 <= col192 <= 1
-    let (col64c, col192) = col64b.overflowing_add(ll_rh);
-    let col192 = if col192 {
-        // col64b and ll_rh have the same sign, and col64c has the opposite sign.
-        if col64b < 0 {
-            -1i128
-        } else {
-            1i128
-        }
-    } else {
-        0i128
-    };
+    // 0 <= col64c <= 2^64 - 1
+    // -2^63 <= col128a <= 2^63 - 1
+    let (col64c, col128a) = i128_lo_hi(col64b);
+
+    // -2^127 + 2^63 <= col64d <= 2^127 - 2^63
+    let col64d = (col64c as i128).wrapping_add(ll_rh);
 
     // 0 <= col64 <= 2^64 - 1
-    // -2^63 <= col128 <= 2^63 - 1
-    let (col64, col128) = i128_lo_hi(col64c);
+    // -2^63 <= col128b <= 2^63 - 1
+    let (col64, col128b) = i128_lo_hi(col64d);
 
     // Since both col0 and col64 fit in 64 bits, ans0 sum will never overflow.
-    let ans0 = col0.wrapping_add((col64 as u128) << 64);
+    let ans0 = (col0 as u128) | ((col64 as u128) << 64);
     // Since lhs * rhs fits in 256 bits, ans128 sum will never overflow.
-    let ans128 = lh_rh.wrapping_add(col128).wrapping_add(col192 << 64);
+    let ans128 = lh_rh
+        .wrapping_add(col128a as i128)
+        .wrapping_add(col128b as i128);
     I256 {
         lo: ans0,
         hi: ans128,
@@ -172,7 +196,7 @@ pub const fn wide_mul_i128(lhs: i128, rhs: i128) -> I256 {
 }
 
 #[inline]
-pub fn shl_u256_max_128(a: U256, sh: u32) -> U256 {
+pub const fn shl_u256_max_128(a: U256, sh: u32) -> U256 {
     if sh == 0 {
         a
     } else if sh == 128 {
@@ -186,7 +210,7 @@ pub fn shl_u256_max_128(a: U256, sh: u32) -> U256 {
 }
 
 #[inline]
-pub fn shl_i256_max_128(a: I256, sh: u32) -> I256 {
+pub const fn shl_i256_max_128(a: I256, sh: u32) -> I256 {
     if sh == 0 {
         a
     } else if sh == 128 {
@@ -202,46 +226,64 @@ pub fn shl_i256_max_128(a: I256, sh: u32) -> I256 {
     }
 }
 
+/// # Safety
+///
+/// d must have msb set.
 #[inline]
-pub fn div_half_u128(r: &mut u128, d: u128, next_half: u128) -> u128 {
+const unsafe fn div_half_u128(r: u128, d: u128, next_half: u64) -> (u128, u128) {
     let (dl, dh) = u128_lo_hi(d);
-    let (mut q, rr) = (*r / dh, *r % dh);
-    let m = q * dl;
-    *r = next_half + (rr << 64);
-    if *r < m {
+    // SAFETY: we know d has the most significant bit set because we normalized
+    if dh == 0 {
+        #[cfg(debug_assertions)]
+        {
+            unreachable!();
+        }
+        #[cfg(not(debug_assertions))]
+        unsafe {
+            hint::unreachable_unchecked();
+        }
+    }
+    let (mut q, rr) = (r / (dh as u128), r % (dh as u128));
+    let m = q * (dl as u128);
+    let mut r = (next_half as u128) + (rr << 64);
+    if r < m {
         q -= 1;
         let (new_r, overflow) = r.overflowing_add(d);
-        *r = if !overflow && new_r < m {
+        r = if !overflow && new_r < m {
             q -= 1;
             new_r.wrapping_add(d)
         } else {
             new_r
         };
     }
-    *r = r.wrapping_sub(m);
-    q
+    r = r.wrapping_sub(m);
+    (r, q)
 }
 
 #[inline]
-pub fn div_rem_u256_u128(mut n: U256, mut d: u128) -> (U256, u128) {
-    assert!(d != 0, "division by zero");
+pub const fn div_rem_u256_u128(mut n: U256, d: NonZeroU128) -> (U256, u128) {
     let zeros = d.leading_zeros();
-    let mut r = if zeros == 0 {
-        0
+    let (mut r, d) = if zeros == 0 {
+        (0, d.get())
     } else {
-        d <<= zeros;
         let n2 = n.hi >> (128 - zeros);
         n.hi = n.hi << zeros | n.lo >> (128 - zeros);
         n.lo <<= zeros;
-        n2
+        (n2, d.get() << zeros)
     };
 
+    // SAFETY: we know that d has msb set because it is not zero and it was
+    // shifted to the right by the number of leading zeros.
     let (nhl, nhh) = u128_lo_hi(n.hi);
-    let qhh = div_half_u128(&mut r, d, nhh);
-    let qhl = div_half_u128(&mut r, d, nhl);
+    let (new_r, qhh) = unsafe { div_half_u128(r, d, nhh) };
+    r = new_r;
+    let (new_r, qhl) = unsafe { div_half_u128(r, d, nhl) };
+    r = new_r;
     let (nll, nlh) = u128_lo_hi(n.lo);
-    let qlh = div_half_u128(&mut r, d, nlh);
-    let qll = div_half_u128(&mut r, d, nll);
+    let (new_r, qlh) = unsafe { div_half_u128(r, d, nlh) };
+    r = new_r;
+    let (new_r, qll) = unsafe { div_half_u128(r, d, nll) };
+    r = new_r;
     let q = U256 {
         lo: qll + (qlh << 64),
         hi: qhl + (qhh << 64),
@@ -250,41 +292,38 @@ pub fn div_rem_u256_u128(mut n: U256, mut d: u128) -> (U256, u128) {
     (q, r)
 }
 
+// must not result in overflow
 #[inline]
-pub fn div_rem_i256_i128(n: I256, d: i128) -> (I256, i128) {
+pub const fn div_rem_i256_i128_no_overflow(n: I256, d: NonZeroI128) -> (I256, i128) {
     let (n_neg, n_abs) = if n.hi < 0 {
         let (nl, overflow) = n.lo.overflowing_neg();
-        let nh = if overflow {
-            !n.hi as u128
-        } else {
-            n.hi.wrapping_neg() as u128
-        };
+        let nh = n.hi.wrapping_neg().wrapping_sub(overflow as i128) as u128;
         (true, U256 { lo: nl, hi: nh })
     } else {
         let nl = n.lo;
         let nh = n.hi as u128;
         (false, U256 { lo: nl, hi: nh })
     };
-    let (d_neg, d_abs) = if d < 0 {
-        (true, d.wrapping_neg() as u128)
+    let (d_neg, d_abs) = if d.get() < 0 {
+        // SAFETY: d and -d are not zero
+        let ud = unsafe { NonZeroU128::new_unchecked(d.get().wrapping_neg() as u128) };
+        (true, ud)
     } else {
-        (false, d as u128)
+        // SAFETY: d is not zero
+        let ud = unsafe { NonZeroU128::new_unchecked(d.get() as u128) };
+        (false, ud)
     };
 
     let (q_abs, r_abs) = div_rem_u256_u128(n_abs, d_abs);
 
-    let q = if n_neg != d_neg {
-        let (ql, overflow) = q_abs.lo.overflowing_neg();
-        let qh = if overflow {
-            !q_abs.hi as i128
-        } else {
-            q_abs.hi.wrapping_neg() as i128
-        };
-        I256 { lo: ql, hi: qh }
-    } else {
+    let q = if n_neg == d_neg {
         let ql = q_abs.lo;
         let qh = q_abs.hi as i128;
         I256 { lo: ql, hi: qh }
+    } else {
+        let (ql, overflow) = q_abs.lo.overflowing_neg();
+        let qh = q_abs.hi.wrapping_neg().wrapping_sub(overflow as u128) as i128;
+        I256 { lo: ql, hi: qh }
     };
     let r = if n_neg {
         r_abs.wrapping_neg() as i128
@@ -295,7 +334,7 @@ pub fn div_rem_i256_i128(n: I256, d: i128) -> (I256, i128) {
 }
 
 #[inline]
-pub fn overflowing_shl_u256_into_u128(a: U256, sh: u32) -> (u128, bool) {
+pub const fn overflowing_shl_u256_into_u128(a: U256, sh: u32) -> (u128, bool) {
     if sh == 128 {
         (a.hi, false)
     } else if sh == 0 {
@@ -308,7 +347,7 @@ pub fn overflowing_shl_u256_into_u128(a: U256, sh: u32) -> (u128, bool) {
 }
 
 #[inline]
-pub fn overflowing_shl_i256_into_i128(a: I256, sh: u32) -> (i128, bool) {
+pub const fn overflowing_shl_i256_into_i128(a: I256, sh: u32) -> (i128, bool) {
     if sh == 128 {
         (a.hi, false)
     } else if sh == 0 {
@@ -325,9 +364,10 @@ pub fn overflowing_shl_i256_into_i128(a: I256, sh: u32) -> (i128, bool) {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use core::num::{NonZeroI128, NonZeroU128};
 
     fn check_udiv_rem(num: U256, den: u128) {
-        let (quot, rem) = div_rem_u256_u128(num, den);
+        let (quot, rem) = div_rem_u256_u128(num, NonZeroU128::new(den).unwrap());
         assert!(rem <= den);
 
         let ql_d = wide_mul_u128(quot.lo, den);
@@ -341,7 +381,7 @@ mod tests {
     }
 
     fn check_idiv_rem_signs(num: I256, den: i128) {
-        let (quot, rem) = div_rem_i256_i128(num, den);
+        let (quot, rem) = div_rem_i256_i128_no_overflow(num, NonZeroI128::new(den).unwrap());
         assert!(rem.unsigned_abs() <= den.unsigned_abs());
 
         if num.hi < 0 {
@@ -350,10 +390,10 @@ mod tests {
             assert!(rem >= 0);
         }
 
-        if (num.hi < 0) != (den < 0) {
-            assert!(quot.hi <= 0);
-        } else {
+        if (num.hi < 0) == (den < 0) {
             assert!(quot.hi >= 0);
+        } else {
+            assert!(quot.hi <= 0);
         }
     }
 
