@@ -1,4 +1,4 @@
-// Copyright © 2018–2022 Trevor Spiteri
+// Copyright © 2018–2023 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -46,11 +46,11 @@ for signed numbers like <code>[FixedI32]\<[U32]></code>, and in the range
 
 In version 1 the [*typenum* crate] is used for the fractional bit count `Frac`;
 the plan is to to have a major version 2 with const generics when the Rust
-compiler’s [`generic_const_expr` feature] is ready and stabilized. An [alpha
+compiler’s [`generic_const_exprs` feature] is ready and stabilized. An [alpha
 version] is already available.
 
-[`generic_const_expr` feature]: https://github.com/rust-lang/rust/issues/76560
-[alpha version]: https://crates.io/crates/fixed/2.0.0-alpha.2
+[`generic_const_exprs` feature]: https://github.com/rust-lang/rust/issues/76560
+[alpha version]: https://docs.rs/fixed/2.0.0-alpha/fixed/
 
 The main features are
 
@@ -98,14 +98,15 @@ The conversions supported cover the following cases.
     [`from_num`] and [`to_num`] methods and [their checked
     versions][`checked_from_num`].
   * Additionally, [`az`] casts are implemented for conversion between
-    fixed-point nubmers and numeric primitives.
+    fixed-point numbers and numeric primitives.
   * Fixed-point numbers can be parsed from decimal strings using [`FromStr`],
     and from binary, octal and hexadecimal strings using the
     [`from_str_binary`], [`from_str_octal`] and [`from_str_hex`] methods. The
     result is rounded to the nearest, with ties rounded to even.
   * Fixed-point numbers can be converted to strings using [`Display`],
-    [`Binary`], [`Octal`], [`LowerHex`] and [`UpperHex`]. The output is rounded
-    to the nearest, with ties rounded to even.
+    [`Binary`], [`Octal`], [`LowerHex`], [`UpperHex`], [`LowerExp`] and
+    [`UpperExp`]. The output is rounded to the nearest, with ties rounded to
+    even.
   * All fixed-point numbers are plain old data, so [`bytemuck`] bit casting
     conversions can be used.
 
@@ -164,17 +165,46 @@ have less fractional bits, so we use [`from_num`] instead.
 
 ## Writing fixed-point constants and values literally
 
-The [*fixed-macro* crate] provides a convenient macro to write down fixed-point
-constants literally in the code.
+The [`lit`] method, which is available as a `const` function, can be used to
+parse literals. It supports
+  * underscores as separators;
+  * prefixes “`0b`”, “`0o`” and “`0x`” for binary, octal and hexadecimal
+    numbers;
+  * an optional decimal exponent with separator “`e`” or “`E`” for decimal,
+    binary and octal numbers, or with separator “`@`” for all supported radices
+    including hexadecimal.
+
+```rust
+use fixed::types::I16F16;
+
+// 0.1275e2 is 12.75
+const TWELVE_POINT_75: I16F16 = I16F16::lit("0.127_5e2");
+// 1.8 hexadecimal is 1.5 decimal, and 18@-1 is 1.8
+const ONE_POINT_5: I16F16 = I16F16::lit("0x_18@-1");
+// 12.75 + 1.5 = 14.25
+let sum = TWELVE_POINT_75 + ONE_POINT_5;
+assert_eq!(sum, 14.25);
+```
+
+The [*fixed-macro* crate] is an alternative which provides a convenient macro to
+write down fixed-point constants literally in the code. It supports underscores
+as separators, binary/octal/hexadecimal integers, and an optional exponent for
+decimal numbers, but it does not support fractions or exponents for
+binary/octal/hexadecimal as they cannot be parsed by the Rust compiler.
 
 ```rust
 # #[cfg(feature = "skip-this-test")] {
 use fixed::types::I16F16;
 use fixed_macro::fixed;
+use fixed_macro::types::I16F16;
 
-const NUM1: I16F16 = fixed!(12.75: I16F16);
-let num2 = NUM1 + fixed!(13.125: I16F16);
-assert_eq!(num2, 25.875);
+// 0.1275e2 is 12.75
+const TWELVE_POINT_75: I16F16 = fixed!(0.127_5e2: I16F16);
+// 11 binary is 3 decimal
+const THREE: I16F16 = I16F16!(0b_11);
+// 12.75 + 3 = 15.75
+let sum = TWELVE_POINT_75 + THREE;
+assert_eq!(sum, 15.75);
 # }
 ```
 
@@ -185,10 +215,10 @@ crate, add it as a dependency inside [*Cargo.toml*]:
 
 ```toml
 [dependencies]
-fixed = "1.15"
+fixed = "1.24"
 ```
 
-The *fixed* crate requires rustc version 1.57.0 or later.
+The *fixed* crate requires rustc version 1.71.0 or later.
 
 ## Optional features
 
@@ -212,7 +242,7 @@ To enable features, you can add the dependency like this to [*Cargo.toml*]:
 
 ```toml
 [dependencies.fixed]
-version = "1.15"
+version = "1.24"
 features = ["serde"]
 ```
 
@@ -283,9 +313,11 @@ shall be dual licensed as above, without any additional terms or conditions.
 [`LosslessTryInto`]: traits::LosslessTryInto
 [`LossyFrom`]: traits::LossyFrom
 [`LossyInto`]: traits::LossyInto
+[`LowerExp`]: core::fmt::LowerExp
 [`LowerHex`]: core::fmt::LowerHex
 [`Octal`]: core::fmt::Octal
 [`U20F12`]: types::U20F12
+[`UpperExp`]: core::fmt::UpperExp
 [`UpperHex`]: core::fmt::UpperHex
 [`az`]: az_crate
 [`bf16`]: half::bf16
@@ -295,12 +327,51 @@ shall be dual licensed as above, without any additional terms or conditions.
 [`from_str_binary`]: FixedI32::from_str_binary
 [`from_str_hex`]: FixedI32::from_str_hex
 [`from_str_octal`]: FixedI32::from_str_octal
+[`lit`]: FixedI32::lit
 [`to_num`]: FixedI32::to_num
 */
 #![cfg_attr(not(feature = "std"), no_std)]
 #![warn(missing_docs)]
 #![warn(unsafe_op_in_unsafe_fn)]
-#![doc(html_root_url = "https://docs.rs/fixed/~1.15")]
+#![doc(html_root_url = "https://docs.rs/fixed/~1.24")]
+#![doc(html_logo_url = "data:image/svg+xml;base64,
+PHN2ZyB3aWR0aD0iMTI4IiBoZWlnaHQ9IjEyOCIgdmVyc2lvbj0iMS4xIiB2aWV3Qm94PSIwIDAgMzMuODY3IDMzLjg2NyIgeG1s
+bnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj48ZGVmcz48Y2xpcFBhdGggaWQ9ImIiPjxjaXJjbGUgY3g9IjE2LjkzMyIg
+Y3k9IjI4MC4wNyIgcj0iMTYuOTMzIiBmaWxsPSIjMDA3MmIyIi8+PC9jbGlwUGF0aD48Y2xpcFBhdGggaWQ9ImEiPjxjaXJjbGUg
+Y3g9IjE2LjkzMyIgY3k9IjI4MC4wNyIgcj0iMTYuOTMzIiBmaWxsPSIjMDA3MmIyIi8+PC9jbGlwUGF0aD48L2RlZnM+PGcgdHJh
+bnNmb3JtPSJ0cmFuc2xhdGUoMCAtMjYzLjEzKSI+PGNpcmNsZSBjeD0iMTYuOTMzIiBjeT0iMjgwLjA3IiByPSIxNi45MzMiIGZp
+bGw9IiNmN2YxYTEiLz48ZyBmaWxsPSIjMDA3MmIyIj48cGF0aCBkPSJtMTUuMzQ2IDI4My41MWgzLjE3NXMwIDAuNzkzNzYgMC41
+MjkxNyAxLjg1MjFoLTQuMjMzM2MwLjUyOTE2LTEuMDU4MyAwLjUyOTE2LTEuODUyMSAwLjUyOTE2LTEuODUyMXoiIHN0cm9rZS13
+aWR0aD0iLjUyOTE3Ii8+PHBhdGggZD0ibTM0LjExMiAyODUuNTRjMi4yODYgMCAzLjgxLTEuMjg2OSAzLjgxLTIuOTgwMyAwLTEu
+NDIyNC0wLjgyOTczLTIuMjUyMS0xLjg2MjctMi44MTA5di0wLjA2NzdjMC43NDUwNy0wLjQ5MTA3IDEuNDA1NS0xLjMyMDggMS40
+MDU1LTIuMzUzNyAwLTEuNzc4LTEuMzAzOS0yLjk0NjQtMy4yNjgxLTIuOTQ2NC0xLjk5ODEgMC0zLjQzNzUgMS4xMzQ1LTMuNDM3
+NSAyLjk2MzMgMCAxLjEzNDUgMC42MDk2IDEuOTEzNSAxLjQzOTMgMi41NHYwLjA2NzdjLTEuMDE2IDAuNTQxODctMS44Mjg4IDEu
+MzM3Ny0xLjgyODggMi42NDE2IDAgMS43NDQxIDEuNTkxNyAyLjk0NjQgMy43NDIzIDIuOTQ2NHptMC42NzczMy02LjQ2ODVjLTEu
+MTAwNy0wLjQyMzMzLTEuODQ1Ny0wLjg0NjY3LTEuODQ1Ny0xLjcyNzIgMC0wLjgyOTczIDAuNTQxODctMS4yMzYxIDEuMjAyMy0x
+LjIzNjEgMC44MTI4IDAgMS4zMDM5IDAuNTU4OCAxLjMwMzkgMS4zODg1IDAgMC41NTg4LTAuMjM3MDcgMS4wODM3LTAuNjYwNCAx
+LjU3NDh6bS0wLjYyNjUzIDQuNzQxM2MtMC44OTc0NiAwLTEuNjU5NS0wLjU1ODgtMS42NTk1LTEuNTA3MSAwLTAuNjYwNCAwLjM1
+NTYtMS4yNyAwLjgyOTczLTEuNzEwMyAxLjM1NDcgMC41NzU3MyAyLjI2OTEgMC45MzEzMyAyLjI2OTEgMS44Nzk2IDAgMC44OTc0
+Ny0wLjYwOTYgMS4zMzc3LTEuNDM5MyAxLjMzNzd6IiBjbGlwLXBhdGg9InVybCgjYikiLz48cGF0aCBkPSJtMjEuMzQ0IDI4NS4z
+NGg3LjU2OTJ2LTIuMDk5N2gtMi40MDQ1Yy0wLjQ5MTA3IDAtMS4yMzYxIDAuMDY3Ny0xLjc5NDkgMC4xMzU0NyAxLjkxMzUtMS44
+Nzk2IDMuNjc0NS0zLjY0MDcgMy42NzQ1LTUuNTg4IDAtMi4wNDg5LTEuNDM5My0zLjQwMzYtMy41NTYtMy40MDM2LTEuNTA3MSAw
+LTIuNTIzMSAwLjU5MjY3LTMuNTU2IDEuNzYxMWwxLjMwMzkgMS4yODY5YzAuNTQxODctMC41NzU3MyAxLjEzNDUtMS4xMDA3IDEu
+OTEzNS0xLjEwMDcgMC45MzEzMyAwIDEuNTI0IDAuNTc1NzQgMS41MjQgMS42MjU2IDAgMS41MDcxLTEuOTY0MyAzLjQzNzUtNC42
+NzM2IDUuODQyeiIvPjxwYXRoIGQ9Im0xNi45MzMgMjg0LjE2YzEuNzI3MiAwIDMuMDE0MS0xLjM1NDcgMy4wMTQxLTMuMTE1NyAw
+LTEuNzk0OS0xLjI4NjktMy4xNDk2LTMuMDE0MS0zLjE0OTYtMS43MjcyIDAtMy4wMTQxIDEuMzU0Ny0zLjAxNDEgMy4xNDk2IDAg
+MS43NjExIDEuMjg2OSAzLjExNTcgMy4wMTQxIDMuMTE1N3oiLz48cGF0aCBkPSJtOC45MTU0IDI4MC4zOGMwLjgxMjggMCAxLjQw
+NTUgMC40MjMzNCAxLjQwNTUgMS41NTc5IDAgMS4yMTkyLTAuNjA5NiAxLjc0NDEtMS4zNTQ3IDEuNzQ0MXMtMS40NTYzLTAuNTQx
+ODYtMS42NzY0LTIuMjM1MmMwLjQ0MDI3LTAuNzYyIDEuMDY2OC0xLjA2NjggMS42MjU2LTEuMDY2OHptMC4xMDE2IDUuMTY0N2Mx
+Ljk0NzMgMCAzLjU3MjktMS4zNzE2IDMuNTcyOS0zLjYwNjggMC0yLjI2OTEtMS4zNTQ3LTMuMzE4OS0zLjIwMDQtMy4zMTg5LTAu
+NjYwNCAwLTEuNTkxNyAwLjQyMzMzLTIuMTUwNSAxLjEzNDUgMC4wODQ2NjctMi41MDYxIDEuMDMyOS0zLjM1MjggMi4yMTgzLTMu
+MzUyOCAwLjYyNjUzIDAgMS4zMDM5IDAuMzU1NiAxLjY3NjQgMC43NjJsMS4zMDM5LTEuNDkwMWMtMC42NzczMy0wLjY5NDI3LTEu
+NzEwMy0xLjI4NjktMy4xMzI3LTEuMjg2OS0yLjI2OTEgMC00LjM1MTkgMS44MTE5LTQuMzUxOSA1LjgyNTEgMCAzLjc3NjEgMS45
+ODEyIDUuMzM0IDQuMDY0IDUuMzM0eiIvPjxwYXRoIGQ9Im0tMC4yMTE2NyAyODUuNTRjMi4zMDI5IDAgMy44NDM5LTEuOTY0MyAz
+Ljg0MzktNS42MjE5cy0xLjU0MDktNS41MzcyLTMuODQzOS01LjUzNzJjLTIuMzAyOSAwLTMuODQzOSAxLjg3OTYtMy44NDM5IDUu
+NTM3MnMxLjU0MDkgNS42MjE5IDMuODQzOSA1LjYyMTl6bTAtMS45MzA0Yy0wLjgyOTczIDAtMS40OTAxLTAuNzYyLTEuNDkwMS0z
+LjY5MTUgMC0yLjk0NjQgMC42NjA0LTMuNjA2OCAxLjQ5MDEtMy42MDY4IDAuODQ2NjcgMCAxLjQ5MDEgMC42NjA0IDEuNDkwMSAz
+LjYwNjggMCAyLjkyOTUtMC42NDM0NyAzLjY5MTUtMS40OTAxIDMuNjkxNXoiIGNsaXAtcGF0aD0idXJsKCNhKSIvPjwvZz48L2c+
+PC9zdmc+Cg==
+")]
 #![doc(test(attr(deny(warnings))))]
 #![cfg_attr(feature = "fail-on-warnings", deny(warnings))]
 
@@ -313,12 +384,15 @@ mod macros;
 mod arith;
 #[cfg(feature = "borsh")]
 mod borshize;
+mod bytes;
 mod cast;
 mod cmp;
+mod cmp_fixed;
 pub mod consts;
 mod convert;
 mod debug_hex;
 mod display;
+pub mod f128;
 mod float_helper;
 mod from_str;
 mod helpers;
@@ -331,8 +405,10 @@ mod int256;
 mod int_helper;
 mod inv_lerp;
 mod lerp;
+mod log;
 mod log10;
 mod prim_traits;
+mod saturating;
 #[cfg(feature = "serde")]
 mod serdeize;
 pub mod traits;
@@ -343,21 +419,22 @@ mod wrapping;
 
 #[cfg(feature = "num-traits")]
 pub use crate::impl_num_traits::RadixParseFixedError;
+pub use crate::{
+    f128::private::F128, from_str::ParseFixedError, saturating::Saturating, unwrapped::Unwrapped,
+    wrapping::Wrapping,
+};
 use crate::{
-    from_str::FromStrRadix,
     traits::{FromFixed, ToFixed},
     types::extra::{
-        Diff, IsLessOrEqual, LeEqU128, LeEqU16, LeEqU32, LeEqU64, LeEqU8, Sum, True, Unsigned, U12,
-        U124, U125, U126, U127, U128, U13, U14, U15, U16, U28, U29, U30, U31, U32, U4, U5, U6, U60,
-        U61, U62, U63, U64, U7, U8,
+        Diff, IsLessOrEqual, LeEqU128, LeEqU16, LeEqU32, LeEqU64, LeEqU8, Sum, True, Unsigned, U0,
+        U12, U124, U125, U126, U127, U128, U13, U14, U15, U16, U28, U29, U30, U31, U32, U4, U5, U6,
+        U60, U61, U62, U63, U64, U7, U8,
     },
 };
-pub use crate::{from_str::ParseFixedError, unwrapped::Unwrapped, wrapping::Wrapping};
+use core::num::{NonZeroU128, NonZeroU16, NonZeroU32, NonZeroU64, NonZeroU8};
 use core::{
-    cmp::Ordering,
     hash::{Hash, Hasher},
     marker::PhantomData,
-    mem,
     ops::{Add, Sub},
 };
 
@@ -414,7 +491,8 @@ macro_rules! fixed {
             $s_nbits_p1:expr, $s_nbits_m1:expr, $s_nbits_m2:expr, $s_nbits_m3:expr, $s_nbits_m4:expr
         ),
         $nbytes:expr, $bytes_val:expr, $rev_bytes_val:expr, $be_bytes:expr, $le_bytes:expr,
-        $IFixed:ident, $UFixed:ident, $IInner:ident, $UInner:ident, $Signedness:tt,
+        $IFixed:ident, $UFixed:ident,
+        $IInner:ident, $UInner:ident, $NonZeroUInner:ident, $Signedness:tt,
         $LeEqU_C0:tt, $LeEqU_C1:tt, $LeEqU_C2:tt, $LeEqU_C3:tt,
         $HasDouble:tt, $s_nbits_2:expr,
         $Double:ident, $DoubleInner:ty, $IDouble:ident, $IDoubleInner:ty
@@ -427,7 +505,7 @@ macro_rules! fixed {
             ),
             $nbytes, $bytes_val, $rev_bytes_val, $be_bytes, $le_bytes,
             $IFixed[stringify!($IFixed)], $UFixed[stringify!($UFixed)],
-            $IInner, $UInner, $Signedness,
+            $IInner, $UInner, $NonZeroUInner, $Signedness,
             $LeEqU_C0, $LeEqU_C1, $LeEqU_C2, $LeEqU_C3,
             $HasDouble, $s_nbits_2,
             $Double[stringify!($Double)], $DoubleInner, $IDouble, $IDoubleInner
@@ -441,7 +519,7 @@ macro_rules! fixed {
         ),
         $nbytes:expr, $bytes_val:expr, $rev_bytes_val:expr, $be_bytes:expr, $le_bytes:expr,
         $IFixed:ident[$s_ifixed:expr], $UFixed:ident[$s_ufixed:expr],
-        $IInner:ident, $UInner:ident, $Signedness:tt,
+        $IInner:ident, $UInner:ident, $NonZeroUInner:ident, $Signedness:tt,
         $LeEqU_C0:tt, $LeEqU_C1:tt, $LeEqU_C2:tt, $LeEqU_C3:tt,
         $HasDouble:tt, $s_nbits_2:expr,
         $Double:ident[$s_double:expr], $DoubleInner:ty, $IDouble:ident, $IDoubleInner:ty
@@ -488,7 +566,7 @@ range ",
 
 `Frac` is an [`Unsigned`] as provided by the [*typenum* crate]; the plan is to
 to have a major version 2 where `Frac` is replaced by `FRAC` of type [`i32`]
-when the Rust compiler’s [`generic_const_expr` feature] is ready and stabilized.
+when the Rust compiler’s [`generic_const_exprs` feature] is ready and stabilized.
 An [alpha version] is already available.
 
 `", $s_fixed, "<Frac>` has the same size, alignment and ABI as [`", $s_inner, "`];
@@ -511,8 +589,8 @@ assert_eq!(two_point_75.to_string(), \"2.8\");
 [*typenum* crate]: https://crates.io/crates/typenum
 [U", $s_nbits, "]: crate::types::extra::U", $s_nbits, "
 [U0]: crate::types::extra::U0
-[`generic_const_expr` feature]: https://github.com/rust-lang/rust/issues/76560
-[alpha version]: https://crates.io/crates/fixed/2.0.0-alpha.2
+[`generic_const_exprs` feature]: https://github.com/rust-lang/rust/issues/76560
+[alpha version]: https://docs.rs/fixed/2.0.0-alpha/fixed/
 ";
             #[repr(transparent)]
             pub struct $Fixed<Frac> {
@@ -524,10 +602,7 @@ assert_eq!(two_point_75.to_string(), \"2.8\");
         impl<Frac> Clone for $Fixed<Frac> {
             #[inline]
             fn clone(&self) -> $Fixed<Frac> {
-                $Fixed {
-                    bits: self.bits,
-                    phantom: PhantomData,
-                }
+                *self
             }
         }
 
@@ -565,7 +640,7 @@ assert_eq!(two_point_75.to_string(), \"2.8\");
         // inherent methods that require Frac bounds, and cannot be const
         fixed_frac! {
             $Fixed[$s_fixed]($Inner[$s_inner], $LeEqU, $s_nbits, $s_nbits_m1, $s_nbits_m4),
-            $UFixed, $UInner, $Signedness
+            $UFixed, $UInner, $NonZeroUInner, $Signedness
         }
         fixed_const! {
             $Fixed[$s_fixed]($LeEqU, $s_nbits, $s_nbits_m1, $s_nbits_m2, $s_nbits_m3, $s_nbits_m4),
@@ -579,7 +654,7 @@ fixed! {
     "An eight",
     FixedU8(u8, LeEqU8, U8, U7, "8", "9", "7", "6", "5", "4"),
     1, "0x12", "0x12", "[0x12]", "[0x12]",
-    FixedI8, FixedU8, i8, u8, Unsigned,
+    FixedI8, FixedU8, i8, u8, NonZeroU8, Unsigned,
     U8, U7, U6, U5,
     True, "16", FixedU16, u16, FixedI16, i16
 }
@@ -587,7 +662,7 @@ fixed! {
     "A 16",
     FixedU16(u16, LeEqU16, U16, U15, "16", "17", "15", "14", "13", "12"),
     2, "0x1234", "0x3412", "[0x12, 0x34]", "[0x34, 0x12]",
-    FixedI16, FixedU16, i16, u16, Unsigned,
+    FixedI16, FixedU16, i16, u16, NonZeroU16, Unsigned,
     U16, U15, U14, U13,
     True, "32", FixedU32, u32, FixedI32, i32
 }
@@ -595,7 +670,7 @@ fixed! {
     "A 32",
     FixedU32(u32, LeEqU32, U32, U31, "32", "33", "31", "30", "29", "28"),
     4, "0x1234_5678", "0x7856_3412", "[0x12, 0x34, 0x56, 0x78]", "[0x78, 0x56, 0x34, 0x12]",
-    FixedI32, FixedU32, i32, u32, Unsigned,
+    FixedI32, FixedU32, i32, u32, NonZeroU32, Unsigned,
     U32, U31, U30, U29,
     True, "64", FixedU64, u64, FixedI64, i64
 }
@@ -605,7 +680,7 @@ fixed! {
     8, "0x1234_5678_9ABC_DE0F", "0x0FDE_BC9A_7856_3412",
     "[0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC, 0xDE, 0x0F]",
     "[0x0F, 0xDE, 0xBC, 0x9A, 0x78, 0x56, 0x34, 0x12]",
-    FixedI64, FixedU64, i64, u64, Unsigned,
+    FixedI64, FixedU64, i64, u64, NonZeroU64, Unsigned,
     U64, U63, U62, U61,
     True, "128", FixedU128, u128, FixedI128, i128
 }
@@ -618,7 +693,7 @@ fixed! {
      0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08]",
     "[0x08, 0x07, 0x06, 0x05, 0x04, 0x03, 0x02, 0x01, \
      0xF0, 0xDE, 0xBC, 0x9A, 0x78, 0x56, 0x34, 0x12]",
-    FixedI128, FixedU128, i128, u128, Unsigned,
+    FixedI128, FixedU128, i128, u128, NonZeroU128, Unsigned,
     U128, U127, U126, U125,
     False, "128", FixedU128, u128, FixedI128, i128
 }
@@ -626,7 +701,7 @@ fixed! {
     "An eight",
     FixedI8(i8, LeEqU8, U8, U7, "8", "9", "7", "6", "5", "4"),
     1, "0x12", "0x12", "[0x12]", "[0x12]",
-    FixedI8, FixedU8, i8, u8, Signed,
+    FixedI8, FixedU8, i8, u8, NonZeroU8, Signed,
     U7, U6, U5, U4,
     True, "16", FixedI16, i16, FixedI16, i16
 }
@@ -634,7 +709,7 @@ fixed! {
     "A 16",
     FixedI16(i16, LeEqU16, U16, U15, "16", "17", "15", "14", "13", "12"),
     2, "0x1234", "0x3412", "[0x12, 0x34]", "[0x34, 0x12]",
-    FixedI16, FixedU16, i16, u16, Signed,
+    FixedI16, FixedU16, i16, u16, NonZeroU16, Signed,
     U15, U14, U13, U12,
     True, "32", FixedI32, i32, FixedI32, i32
 }
@@ -642,7 +717,7 @@ fixed! {
     "A 32",
     FixedI32(i32, LeEqU32, U32, U31, "32", "33", "31", "30", "29", "28"),
     4, "0x1234_5678", "0x7856_3412", "[0x12, 0x34, 0x56, 0x78]", "[0x78, 0x56, 0x34, 0x12]",
-    FixedI32, FixedU32, i32, u32, Signed,
+    FixedI32, FixedU32, i32, u32, NonZeroU32, Signed,
     U31, U30, U29, U28,
     True, "64", FixedI64, i64, FixedI64, i64
 }
@@ -652,7 +727,7 @@ fixed! {
     8, "0x1234_5678_9ABC_DE0F", "0x0FDE_BC9A_7856_3412",
     "[0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC, 0xDE, 0x0F]",
     "[0x0F, 0xDE, 0xBC, 0x9A, 0x78, 0x56, 0x34, 0x12]",
-    FixedI64, FixedU64, i64, u64, Signed,
+    FixedI64, FixedU64, i64, u64, NonZeroU64, Signed,
     U63, U62, U61, U60,
     True, "128", FixedI128, i128, FixedI128, i128
 }
@@ -665,7 +740,7 @@ fixed! {
      0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08]",
     "[0x08, 0x07, 0x06, 0x05, 0x04, 0x03, 0x02, 0x01, \
      0xF0, 0xDE, 0xBC, 0x9A, 0x78, 0x56, 0x34, 0x12]",
-    FixedI128, FixedU128, i128, u128, Signed,
+    FixedI128, FixedU128, i128, u128, NonZeroU128, Signed,
     U127, U126, U125, U124,
     False, "128", FixedI128, i128, FixedI128, i128
 }
@@ -679,24 +754,24 @@ fixed! {
 ///   * compare fixed-point numbers and the bit representation of 128-bit
 ///     floating-point numbers.
 ///
-/// # Examples
+/// This is deprecated, and [`F128`] should be used instead. There are two main
+/// differences to keep in mind when switching to [`F128`]:
 ///
-/// ```rust
-/// use fixed::{types::I16F16, F128Bits};
-/// // binary128 representation for 1.0 is 0x3FFF << 112
-/// let one = F128Bits(0x3FFF_u128 << 112);
+///   * The ordering for `F128Bits` is total ordering, not regular
+///     floating-point number ordering, while the ordering for [`F128`] is
+///     similar to ordering for standard floating-point numbers.
+///   * The underlying [`u128`] value for `F128Bits` is accessible as a public
+///     field, while for [`F128`] it is accessible only through the [`to_bits`]
+///     and [`from_bits`] methods.
 ///
-/// assert_eq!(I16F16::ONE.to_num::<F128Bits>(), one);
-/// assert_eq!(I16F16::from_num(one), I16F16::ONE);
-///
-/// // fixed-point numbers can be compared directly to F128Bits values
-/// assert!(I16F16::from_num(1.5) > one);
-/// assert!(I16F16::from_num(0.5) < one);
-/// ```
+/// [`from_bits`]: F128::from_bits
+/// [`to_bits`]: F128::to_bits
+#[deprecated(since = "1.18.0", note = "use `F128` instead")]
 #[repr(transparent)]
 #[derive(Clone, Copy, Default, Hash, Debug, Eq, PartialEq, Ord, PartialOrd)]
 pub struct F128Bits(pub u128);
 
+#[allow(deprecated)]
 impl F128Bits {
     #[inline]
     pub(crate) fn to_bits(self) -> u128 {
@@ -707,11 +782,6 @@ impl F128Bits {
     pub(crate) fn from_bits(bits: u128) -> F128Bits {
         F128Bits(bits)
     }
-
-    #[inline]
-    pub(crate) fn is_nan(self) -> bool {
-        (self.to_bits() & !float_helper::F128Bits::SIGN_MASK) > float_helper::F128Bits::EXP_MASK
-    }
 }
 
 /// Defines constant fixed-point numbers from integer expressions.
@@ -732,6 +802,7 @@ impl F128Bits {
 /// # Examples
 ///
 /// ```rust
+/// # #![allow(deprecated)]
 /// use fixed::{const_fixed_from_int, types::I16F16};
 /// const_fixed_from_int! {
 ///     // define a constant using an integer
@@ -747,6 +818,7 @@ impl F128Bits {
 /// <code>[i32]::[MAX][i32::MAX]</code> is not representable by [`I16F16`].
 ///
 /// ```rust,compile_fail
+/// # #![allow(deprecated)]
 /// use fixed::{const_fixed_from_int, types::I16F16};
 /// const_fixed_from_int! {
 ///     // fails because i32::MAX > I16F16::MAX
@@ -759,6 +831,7 @@ impl F128Bits {
 /// constants using [`i32`] expressions, not [`i16`] expressions.
 ///
 /// ```rust,compile_fail
+/// # #![allow(deprecated)]
 /// use fixed::{const_fixed_from_int, types::I16F16};
 /// const_fixed_from_int! {
 ///     // fails because 0i16 is not of type i32
@@ -770,6 +843,7 @@ impl F128Bits {
 /// [`I16F16`]: crate::types::I16F16
 /// [`from_num`]: FixedI32::from_num
 #[macro_export]
+#[deprecated(since = "1.20.0", note = "use the `const_from_int` method instead")]
 macro_rules! const_fixed_from_int {
     ($($vis:vis const $NAME:ident: $Fixed:ty = $int:expr;)*) => { $(
         $vis const $NAME: $Fixed = <$Fixed>::from_bits({
@@ -790,31 +864,27 @@ macro_rules! const_fixed_from_int {
 /// The first snippet succeeds, and acts as a control.
 ///
 /// ```rust
-/// use fixed::{const_fixed_from_int, types::*};
-/// const_fixed_from_int! {
-///     const ZERO_I0: I0F32 = 0;
-///     const ZERO_I1: I32F0 = 0;
-///     const ZERO_U0: U0F32 = 0;
-///     const ZERO_U1: U32F0 = 0;
+/// use fixed::types::*;
+/// const ZERO_I0: I0F32 = I0F32::const_from_int(0);
+/// const ZERO_I1: I32F0 = I32F0::const_from_int(0);
+/// const ZERO_U0: U0F32 = U0F32::const_from_int(0);
+/// const ZERO_U1: U32F0 = U32F0::const_from_int(0);
 ///
-///     const ONE_I0: I2F30 = 1;
-///     const ONE_I1: I32F0 = 1;
-///     const ONE_U0: U1F31 = 1;
-///     const ONE_U1: U32F0 = 1;
+/// const ONE_I0: I2F30 = I2F30::const_from_int(1);
+/// const ONE_I1: I32F0 = I32F0::const_from_int(1);
+/// const ONE_U0: U1F31 = U1F31::const_from_int(1);
+/// const ONE_U1: U32F0 = U32F0::const_from_int(1);
 ///
-///     const MINUS_ONE_I0: I1F31 = -1;
-///     const MINUS_ONE_I1: I32F0 = -1;
+/// const MINUS_ONE_I0: I1F31 = I1F31::const_from_int(-1);
+/// const MINUS_ONE_I1: I32F0 = I32F0::const_from_int(-1);
 ///
-///     const MINUS_TWO_I0: I2F30 = -2;
-///     const MINUS_TWO_I1: I32F0 = -2;
-/// }
+/// const MINUS_TWO_I0: I2F30 = I2F30::const_from_int(-2);
+/// const MINUS_TWO_I1: I32F0 = I32F0::const_from_int(-2);
 ///
 /// mod test_pub {
-///     use fixed::{const_fixed_from_int, types::*};
+///     use fixed::types::*;
 ///
-///     const_fixed_from_int! {
-///         pub const PUB: I32F0 = 0;
-///     }
+///     pub const PUB: I32F0 = I32F0::const_from_int(0);
 /// }
 ///
 /// assert_eq!(ZERO_I0, 0);
@@ -840,38 +910,28 @@ macro_rules! const_fixed_from_int {
 ///
 /// Not enough integer bits for 1.
 /// ```rust,compile_fail
-/// use fixed::{const_fixed_from_int, types::*};
-/// const_fixed_from_int! {
-///     const _ONE: I0F32 = 1;
-/// }
+/// use fixed::types::*;
+/// const _ONE: I0F32 = I0F32::const_from_int(1);
 /// ```
 /// ```rust,compile_fail
-/// use fixed::{const_fixed_from_int, types::*};
-/// const_fixed_from_int! {
-///     const _ONE: I1F31 = 1;
-/// }
+/// use fixed::types::*;
+/// const _ONE: I1F31 = I1F31::const_from_int(1);
 /// ```
 /// ```rust,compile_fail
-/// use fixed::{const_fixed_from_int, types::*};
-/// const_fixed_from_int! {
-///     const _ONE: U0F32 = 1;
-/// }
+/// use fixed::types::*;
+/// const _ONE: U0F32 = U0F32::const_from_int(1);
 /// ```
 ///
 /// Not enough integer bits for -1.
 /// ```rust,compile_fail
-/// use fixed::{const_fixed_from_int, types::*};
-/// const_fixed_from_int! {
-///     const _MINUS_ONE: I0F32 = -1;
-/// }
+/// use fixed::types::*;
+/// const _MINUS_ONE: I0F32 = I0F32::const_from_int(-1);
 /// ```
 ///
 /// Not enough integer bits for -2.
 /// ```rust,compile_fail
-/// use fixed::{const_fixed_from_int, types::*};
-/// const_fixed_from_int! {
-///     const _MINUS_TWO: I1F31 = -2;
-/// }
+/// use fixed::types::*;
+/// const _MINUS_TWO: I1F31 = I1F31::const_from_int(-2);
 /// ```
 fn _compile_fail_tests() {}
 
@@ -921,33 +981,27 @@ mod tests {
         let f = I1F31::from_bits((-1) << 31);
         assert_eq!(f.to_num::<i32>(), -1);
         assert_eq!(f.round_to_zero(), -1);
-        assert_eq!(f.overflowing_ceil(), (I1F31::from_num(-1), false));
-        assert_eq!(f.overflowing_floor(), (I1F31::from_num(-1), false));
-        assert_eq!(f.overflowing_round(), (I1F31::from_num(-1), false));
-        assert_eq!(
-            f.overflowing_round_ties_to_even(),
-            (I1F31::from_num(-1), false)
-        );
+        assert_eq!(f.overflowing_ceil(), (I1F31::NEG_ONE, false));
+        assert_eq!(f.overflowing_floor(), (I1F31::NEG_ONE, false));
+        assert_eq!(f.overflowing_round(), (I1F31::NEG_ONE, false));
+        assert_eq!(f.overflowing_round_ties_to_even(), (I1F31::NEG_ONE, false));
 
         // -0.5 - Δ
         let f = I1F31::from_bits(((-1) << 30) - 1);
         assert_eq!(f.to_num::<i32>(), -1);
         assert_eq!(f.round_to_zero(), 0);
         assert_eq!(f.overflowing_ceil(), (I1F31::ZERO, false));
-        assert_eq!(f.overflowing_floor(), (I1F31::from_num(-1), false));
-        assert_eq!(f.overflowing_round(), (I1F31::from_num(-1), false));
-        assert_eq!(
-            f.overflowing_round_ties_to_even(),
-            (I1F31::from_num(-1), false)
-        );
+        assert_eq!(f.overflowing_floor(), (I1F31::NEG_ONE, false));
+        assert_eq!(f.overflowing_round(), (I1F31::NEG_ONE, false));
+        assert_eq!(f.overflowing_round_ties_to_even(), (I1F31::NEG_ONE, false));
 
         // -0.5
         let f = I1F31::from_bits((-1) << 30);
         assert_eq!(f.to_num::<i32>(), -1);
         assert_eq!(f.round_to_zero(), 0);
         assert_eq!(f.overflowing_ceil(), (I1F31::ZERO, false));
-        assert_eq!(f.overflowing_floor(), (I1F31::from_num(-1), false));
-        assert_eq!(f.overflowing_round(), (I1F31::from_num(-1), false));
+        assert_eq!(f.overflowing_floor(), (I1F31::NEG_ONE, false));
+        assert_eq!(f.overflowing_round(), (I1F31::NEG_ONE, false));
         assert_eq!(f.overflowing_round_ties_to_even(), (I1F31::ZERO, false));
 
         // -0.5 + Δ
@@ -955,7 +1009,7 @@ mod tests {
         assert_eq!(f.to_num::<i32>(), -1);
         assert_eq!(f.round_to_zero(), 0);
         assert_eq!(f.overflowing_ceil(), (I1F31::ZERO, false));
-        assert_eq!(f.overflowing_floor(), (I1F31::from_num(-1), false));
+        assert_eq!(f.overflowing_floor(), (I1F31::NEG_ONE, false));
         assert_eq!(f.overflowing_round(), (I1F31::ZERO, false));
         assert_eq!(f.overflowing_round_ties_to_even(), (I1F31::ZERO, false));
 
@@ -963,7 +1017,7 @@ mod tests {
         let f = I1F31::from_bits((1 << 30) - 1);
         assert_eq!(f.to_num::<i32>(), 0);
         assert_eq!(f.round_to_zero(), 0);
-        assert_eq!(f.overflowing_ceil(), (I1F31::from_num(-1), true));
+        assert_eq!(f.overflowing_ceil(), (I1F31::NEG_ONE, true));
         assert_eq!(f.overflowing_floor(), (I1F31::ZERO, false));
         assert_eq!(f.overflowing_round(), (I1F31::ZERO, false));
         assert_eq!(f.overflowing_round_ties_to_even(), (I1F31::ZERO, false));
@@ -972,9 +1026,9 @@ mod tests {
         let f = I1F31::from_bits(1 << 30);
         assert_eq!(f.to_num::<i32>(), 0);
         assert_eq!(f.round_to_zero(), 0);
-        assert_eq!(f.overflowing_ceil(), (I1F31::from_num(-1), true));
+        assert_eq!(f.overflowing_ceil(), (I1F31::NEG_ONE, true));
         assert_eq!(f.overflowing_floor(), (I1F31::ZERO, false));
-        assert_eq!(f.overflowing_round(), (I1F31::from_num(-1), true));
+        assert_eq!(f.overflowing_round(), (I1F31::NEG_ONE, true));
         assert_eq!(f.overflowing_round_ties_to_even(), (I1F31::ZERO, false));
 
         // 0
@@ -990,13 +1044,10 @@ mod tests {
         let f = I1F31::from_bits((1 << 30) + 1);
         assert_eq!(f.to_num::<i32>(), 0);
         assert_eq!(f.round_to_zero(), 0);
-        assert_eq!(f.overflowing_ceil(), (I1F31::from_num(-1), true));
+        assert_eq!(f.overflowing_ceil(), (I1F31::NEG_ONE, true));
         assert_eq!(f.overflowing_floor(), (I1F31::ZERO, false));
-        assert_eq!(f.overflowing_round(), (I1F31::from_num(-1), true));
-        assert_eq!(
-            f.overflowing_round_ties_to_even(),
-            (I1F31::from_num(-1), true)
-        );
+        assert_eq!(f.overflowing_round(), (I1F31::NEG_ONE, true));
+        assert_eq!(f.overflowing_round_ties_to_even(), (I1F31::NEG_ONE, true));
 
         // -3.5 - Δ
         let f = I16F16::from_bits(((-7) << 15) - 1);
@@ -1074,33 +1125,27 @@ mod tests {
         let f = I16F16::from_bits((-1) << 16);
         assert_eq!(f.to_num::<i32>(), -1);
         assert_eq!(f.round_to_zero(), -1);
-        assert_eq!(f.overflowing_ceil(), (I16F16::from_num(-1), false));
-        assert_eq!(f.overflowing_floor(), (I16F16::from_num(-1), false));
-        assert_eq!(f.overflowing_round(), (I16F16::from_num(-1), false));
-        assert_eq!(
-            f.overflowing_round_ties_to_even(),
-            (I16F16::from_num(-1), false)
-        );
+        assert_eq!(f.overflowing_ceil(), (I16F16::NEG_ONE, false));
+        assert_eq!(f.overflowing_floor(), (I16F16::NEG_ONE, false));
+        assert_eq!(f.overflowing_round(), (I16F16::NEG_ONE, false));
+        assert_eq!(f.overflowing_round_ties_to_even(), (I16F16::NEG_ONE, false));
 
         // -0.5 - Δ
         let f = I16F16::from_bits(((-1) << 15) - 1);
         assert_eq!(f.to_num::<i32>(), -1);
         assert_eq!(f.round_to_zero(), 0);
         assert_eq!(f.overflowing_ceil(), (I16F16::ZERO, false));
-        assert_eq!(f.overflowing_floor(), (I16F16::from_num(-1), false));
-        assert_eq!(f.overflowing_round(), (I16F16::from_num(-1), false));
-        assert_eq!(
-            f.overflowing_round_ties_to_even(),
-            (I16F16::from_num(-1), false)
-        );
+        assert_eq!(f.overflowing_floor(), (I16F16::NEG_ONE, false));
+        assert_eq!(f.overflowing_round(), (I16F16::NEG_ONE, false));
+        assert_eq!(f.overflowing_round_ties_to_even(), (I16F16::NEG_ONE, false));
 
         // -0.5
         let f = I16F16::from_bits((-1) << 15);
         assert_eq!(f.to_num::<i32>(), -1);
         assert_eq!(f.round_to_zero(), 0);
         assert_eq!(f.overflowing_ceil(), (I16F16::ZERO, false));
-        assert_eq!(f.overflowing_floor(), (I16F16::from_num(-1), false));
-        assert_eq!(f.overflowing_round(), (I16F16::from_num(-1), false));
+        assert_eq!(f.overflowing_floor(), (I16F16::NEG_ONE, false));
+        assert_eq!(f.overflowing_round(), (I16F16::NEG_ONE, false));
         assert_eq!(f.overflowing_round_ties_to_even(), (I16F16::ZERO, false));
 
         // -0.5 + Δ
@@ -1108,7 +1153,7 @@ mod tests {
         assert_eq!(f.to_num::<i32>(), -1);
         assert_eq!(f.round_to_zero(), 0);
         assert_eq!(f.overflowing_ceil(), (I16F16::ZERO, false));
-        assert_eq!(f.overflowing_floor(), (I16F16::from_num(-1), false));
+        assert_eq!(f.overflowing_floor(), (I16F16::NEG_ONE, false));
         assert_eq!(f.overflowing_round(), (I16F16::ZERO, false));
         assert_eq!(f.overflowing_round_ties_to_even(), (I16F16::ZERO, false));
 
@@ -1438,10 +1483,7 @@ mod tests {
             (I1F31::ZERO, true)
         );
         // -1 does not overflow
-        assert_eq!(
-            I1F31::from_num(-1).overflowing_recip(),
-            (I1F31::from_num(-1), false)
-        );
+        assert_eq!(I1F31::NEG_ONE.overflowing_recip(), (I1F31::NEG_ONE, false));
     }
 
     #[test]
