@@ -1,4 +1,4 @@
-// Copyright © 2018–2022 Trevor Spiteri
+// Copyright © 2018–2023 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -14,7 +14,12 @@
 // <https://opensource.org/licenses/MIT>.
 
 macro_rules! fixed_from_to {
-    ($Fixed:ident[$s_fixed:expr]($Inner:ty[$s_inner:expr], $s_nbits:expr), $Signedness:tt) => {
+    (
+        $Fixed:ident[$s_fixed:expr](
+            $Inner:ident[$s_inner:expr], $LeEqU:tt, $s_nbits:expr
+        ),
+        $Signedness:tt
+    ) => {
         comment! {
             r#"Creates a fixed-point number from another number.
 
@@ -26,7 +31,7 @@ The other number can be:
     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     [`usize`].
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds
+    [`f64`] or [`F128`]. For this conversion, the method rounds
     to the nearest, with ties rounding to even.
   * Any other number `src` for which [`ToFixed`] is implemented, in
     which case this method returns
@@ -76,6 +81,7 @@ assert_eq!(Fix::from_num(",
 [finite]: f64::is_finite
 ";
             #[inline]
+            #[track_caller]
             pub fn from_num<Src: ToFixed>(src: Src) -> $Fixed<Frac> {
                 src.to_fixed()
             }
@@ -93,7 +99,7 @@ The other number can be:
     [`usize`]. Any fractional bits are discarded, which rounds towards
     &minus;∞.
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other type `Dst` for which [`FromFixed`] is implemented, in
     which case this method returns
@@ -146,6 +152,8 @@ assert_eq!(",
 [`wrapping_to_num`]: Self::wrapping_to_num
 ";
             #[inline]
+            #[track_caller]
+            #[must_use]
             pub fn to_num<Dst: FromFixed>(self) -> Dst {
                 Dst::from_fixed(self)
             }
@@ -163,7 +171,7 @@ The other number can be:
     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     [`usize`].
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other number `src` for which [`ToFixed`] is implemented, in
     which case this method returns
@@ -233,7 +241,7 @@ The other number can be:
     [`usize`]. Any fractional bits are discarded, which rounds towards
     &minus;∞.
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other type `Dst` for which [`FromFixed`] is implemented, in
     which case this method returns
@@ -284,6 +292,7 @@ assert_eq!(one_point_625.checked_to_num::<f32>(), Some(1.625f32));
 [`f16`]: half::f16
 ";
             #[inline]
+            #[must_use]
             pub fn checked_to_num<Dst: FromFixed>(self) -> Option<Dst> {
                 Dst::checked_from_fixed(self)
             }
@@ -301,7 +310,7 @@ The other number can be:
     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     [`usize`].
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other number `src` for which [`ToFixed`] is implemented, in
     which case this method returns
@@ -356,6 +365,7 @@ assert_eq!(Fix::saturating_from_num(std::f64::NEG_INFINITY), Fix::MIN);
 [`f16`]: half::f16
 ";
             #[inline]
+            #[track_caller]
             pub fn saturating_from_num<Src: ToFixed>(src: Src) -> $Fixed<Frac> {
                 src.saturating_to_fixed()
             }
@@ -374,7 +384,7 @@ The other number can be:
     [`usize`]. Any fractional bits are discarded, which rounds towards
     &minus;∞.
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other type `Dst` for which [`FromFixed`] is implemented, in
     which case this method returns
@@ -422,6 +432,7 @@ assert_eq!(one_point_625.saturating_to_num::<f32>(), 1.625f32);
 [`f16`]: half::f16
 ";
             #[inline]
+            #[must_use]
             pub fn saturating_to_num<Dst: FromFixed>(self) -> Dst {
                 Dst::saturating_from_fixed(self)
             }
@@ -439,7 +450,7 @@ The other number can be:
     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     [`usize`].
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other number `src` for which [`ToFixed`] is implemented, in
     which case this method returns
@@ -486,6 +497,7 @@ assert_eq!(Fix::wrapping_from_num(large), wrapped);
 [finite]: f64::is_finite
 ";
             #[inline]
+            #[track_caller]
             pub fn wrapping_from_num<Src: ToFixed>(src: Src) -> $Fixed<Frac> {
                 src.wrapping_to_fixed()
             }
@@ -504,7 +516,7 @@ The other number can be:
     [`usize`]. Any fractional bits are discarded, which rounds towards
     &minus;∞.
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other type `Dst` for which [`FromFixed`] is implemented, in
     which case this method returns
@@ -552,6 +564,7 @@ assert_eq!(one_point_625.wrapping_to_num::<f32>(), 1.625f32);
 [`f16`]: half::f16
 ";
             #[inline]
+            #[must_use]
             pub fn wrapping_to_num<Dst: FromFixed>(self) -> Dst {
                 Dst::wrapping_from_fixed(self)
             }
@@ -569,7 +582,7 @@ The other number can be:
     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     [`usize`].
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other number `src` for which [`ToFixed`] is implemented, in
     which case this method returns
@@ -634,7 +647,7 @@ The other number can be:
     [`usize`]. Any fractional bits are discarded, which rounds towards
     &minus;∞.
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other type `Dst` for which [`FromFixed`] is implemented, in
     which case this method returns
@@ -676,6 +689,7 @@ let _overflow = Fix::MAX.unwrapped_to_num::<TooFewIntBits>();
 ";
             #[inline]
             #[track_caller]
+            #[must_use]
             pub fn unwrapped_to_num<Dst: FromFixed>(self) -> Dst {
                 match Dst::overflowing_from_fixed(self) {
                     (_, true) => panic!("overflow"),
@@ -699,7 +713,7 @@ The other number can be:
     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     [`usize`].
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other number `src` for which [`ToFixed`] is implemented, in
     which case this method returns
@@ -748,6 +762,7 @@ assert_eq!(Fix::overflowing_from_num(large), (wrapped, true));
 [finite]: f64::is_finite
 ";
             #[inline]
+            #[track_caller]
             pub fn overflowing_from_num<Src: ToFixed>(src: Src) -> ($Fixed<Frac>, bool) {
                 src.overflowing_to_fixed()
             }
@@ -768,7 +783,7 @@ The other number can be:
     [`usize`]. Any fractional bits are discarded, which rounds towards
     &minus;∞.
   * A floating-point number of type [`f16`], [`bf16`], [`f32`],
-    [`f64`] or [`F128Bits`]. For this conversion, the method rounds to
+    [`f64`] or [`F128`]. For this conversion, the method rounds to
     the nearest, with ties rounding to even.
   * Any other type `Dst` for which [`FromFixed`] is implemented, in
     which case this method returns
@@ -818,37 +833,286 @@ assert_eq!(one_point_625.overflowing_to_num::<f32>(), (1.625f32, false));
 [`f16`]: half::f16
 ";
             #[inline]
+            #[must_use]
             pub fn overflowing_to_num<Dst: FromFixed>(self) -> (Dst, bool) {
                 Dst::overflowing_from_fixed(self)
             }
         }
 
+        /// Creates a fixed-point number from a fixed-point number with the same
+        /// underlying integer type. Usable in constant context.
+        ///
+        /// This is equivalent to the [`unwrapped_from_num`] method with
+        #[doc = concat!("<code>[", $s_fixed, "]&lt;SrcFrac></code>")]
+        /// as its generic parameter, but can also be used in constant context.
+        /// Unless required in constant context, use [`unwrapped_from_num`] or
+        /// [`from_num`] instead.
+        ///
+        /// # Planned deprecation
+        ///
+        /// This method will be deprecated when the [`unwrapped_from_num`]
+        /// method is usable in constant context.
+        ///
+        /// # Panics
+        ///
+        /// Panics if the value does not fit.
+        ///
+        /// # Examples
+        ///
+        /// ```rust
+        /// use fixed::types::extra::{U2, U4};
+        #[doc = concat!("use fixed::", $s_fixed, ";")]
+        #[doc = concat!("type FixA = ", $s_fixed, "<U2>;")]
+        #[doc = concat!("type FixB = ", $s_fixed, "<U4>;")]
+        /// const A: FixA = FixA::unwrapped_from_str("3.5");
+        /// const B: FixB = FixB::const_from_fixed(A);
+        /// assert_eq!(B, 3.5);
+        /// ```
+        ///
+        /// The following would fail to compile because of overflow.
+        ///
+        /// ```rust,compile_fail
+        /// use fixed::types::extra::{U2, U4};
+        #[doc = concat!("use fixed::", $s_fixed, ";")]
+        #[doc = concat!("const _OVERFLOW: ", $s_fixed, "<U4> = ", $s_fixed, "::const_from_fixed(", $s_fixed, "::<U2>::MAX);")]
+        /// ```
+        ///
+        /// [`from_num`]: Self::from_num
+        /// [`unwrapped_from_num`]: Self::unwrapped_from_num
+        #[inline]
+        #[track_caller]
+        #[must_use]
+        pub const fn const_from_fixed<SrcFrac: $LeEqU>(src: $Fixed<SrcFrac>) -> $Fixed<Frac> {
+            let shift_left = Frac::I32 - SrcFrac::I32;
+            let nbits = $Inner::BITS as i32;
+            let src_bits = src.to_bits();
+            let bits = if shift_left <= -nbits {
+                src_bits >> (nbits / 2) >> (nbits / 2)
+            } else if shift_left <= 0 {
+                src_bits >> -shift_left
+            } else if shift_left >= nbits {
+                if src_bits != 0 {
+                    panic!("overflow");
+                }
+                0
+            } else {
+                let shifted = src_bits << shift_left;
+                if (shifted >> shift_left) != src_bits {
+                    panic!("overflow");
+                }
+                shifted
+            };
+            Self::from_bits(bits)
+        }
+
+        /// Creates a fixed-point number from the underlying integer type
+        #[doc = concat!("[`", $s_inner, "`].")]
+        /// Usable in constant context.
+        ///
+        /// This is equivalent to the [`unwrapped_from_num`] method with
+        #[doc = concat!("[`", $s_inner, "`]")]
+        /// as its generic parameter, but can also be used in constant context.
+        /// Unless required in constant context, use [`unwrapped_from_num`] or
+        /// [`from_num`] instead.
+        ///
+        /// # Planned deprecation
+        ///
+        /// This method will be deprecated when the [`unwrapped_from_num`]
+        /// method is usable in constant context.
+        ///
+        /// # Panics
+        ///
+        /// Panics if the value does not fit.
+        ///
+        /// # Examples
+        ///
+        /// ```rust
+        #[doc = concat!("use fixed::{types::extra::U4, ", $s_fixed, "};")]
+        #[doc = concat!("type Fix = ", $s_fixed, "<U4>;")]
+        /// const FIVE: Fix = Fix::const_from_int(5);
+        /// assert_eq!(FIVE, 5);
+        /// ```
+        ///
+        /// The following would fail to compile because of overflow.
+        ///
+        /// ```rust,compile_fail
+        #[doc = concat!("use fixed::{types::extra::U4, ", $s_fixed, "};")]
+        #[doc = concat!("const _OVERFLOW: ", $s_fixed, "<U4> = ", $s_fixed, "::const_from_int(", $s_inner, "::MAX);")]
+        /// ```
+        ///
+        /// [`from_num`]: Self::from_num
+        /// [`unwrapped_from_num`]: Self::unwrapped_from_num
+        #[inline]
+        #[track_caller]
+        #[must_use]
+        pub const fn const_from_int(src: $Inner) -> $Fixed<Frac> {
+            Self::const_from_fixed($Fixed::<U0>::from_bits(src))
+        }
+
+        comment! {
+            "Parses a fixed-point literal.
+
+Rounding is to the nearest, with ties rounded to even.
+
+This is similar to [`from_str`][Self::from_str] but accepts a prefix for setting
+the radix, and ignores underscores, such that the parsing is more similar to
+numeric literals in Rust code.
+
+",
+            if_signed_else_empty_str! {
+                $Signedness;
+                "The string can start with “`-`” for a negative number.
+
+",
+            },
+            r#"Strings starting with “`0b`” are parsed as binary, strings
+starting with “`0o`” are parsed as octal, and strings starting with “`0x`” are
+parsed as hexadecimal.
+
+Exponents are supported as well. For decimal, binary and octal numbers, the
+separator “`e`” or “`E`” can be used to start an exponent, which is then
+followed by an optional sign “`+`” or “`-`”, and then by a decimal integer which
+is the exponent. For hexadecimal numbers, since “`e`” and “`E`” are hexadecimal
+digits, the separator “`@`” has to be used instead. The separator “`@`” is
+accepted for all radices. The parsed value is scaled by the radix to the power
+of the exponent.
+
+For binary, octal and hexadecimal, base-2 exponents are supported too, using the
+separator “`p`” or “`P`”. The parsed value is scaled by 2 to the power of the
+exponent. For example, for hexadecimal “`P8`” means ×2⁸, and is equivalent to
+“`@2`” which means ×16².
+
+# Panics
+
+Panics if the number is not valid or overflows.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, "#, $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+
+assert_eq!(Fix::lit("1.75"), 1.75);
+assert_eq!(Fix::lit("1_.7_5_"), 1.75);
+assert_eq!(Fix::lit("17.5e-1"), 1.75);
+assert_eq!(Fix::lit("0_.017_5_e+0_2"), 1.75);
+"#,
+            if_signed_else_empty_str! {
+                $Signedness;
+                r#"assert_eq!(Fix::lit("-01.75"), -1.75);
+"#,
+            },
+            r#"
+assert_eq!(Fix::lit("0b_1.1_1"), 1.75);
+assert_eq!(Fix::lit("0b_111e-2"), 1.75);
+
+assert_eq!(Fix::lit("0o_1.6"), 1.75);
+assert_eq!(Fix::lit("0o_.16E1"), 1.75);
+
+assert_eq!(Fix::lit("0x1.C"), 1.75);
+assert_eq!(Fix::lit("0x0.1C@1"), 1.75);
+"#,
+            if_signed_else_empty_str! {
+                $Signedness;
+                r#"assert_eq!(Fix::lit("-0x1.C"), -1.75);
+"#,
+            },
+            r#"```
+
+This method is useful to write constant fixed-point literals.
+
+```rust
+use fixed::{types::extra::U4, "#, $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+const ONE_AND_HALF: Fix = Fix::lit("1.5");
+assert_eq!(ONE_AND_HALF, 1.5);
+```
+"#;
+            #[inline]
+            #[track_caller]
+            #[must_use]
+            pub const fn lit(src: &str) -> $Fixed<Frac> {
+                match from_str::$Inner::lit(src, Self::FRAC_NBITS) {
+                    Ok(s) => $Fixed::from_bits(s),
+                    Err(e) => panic!("{}", e.lit_message()),
+                }
+            }
+        }
+
+        comment! {
+            "Parses a string slice containing decimal digits to return a fixed-point number.
+
+Rounding is to the nearest, with ties rounded to even.
+
+The number can have an optional exponent. The separator “`e`”, “`E`” or “`@`”
+can be used to start an exponent, which is then followed by an optional sign
+“`+`” or “`-`”, and then by a decimal integer which is the exponent. The parsed
+value is scaled by 10 to the power of the exponent.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+assert_eq!(Fix::from_str("1.75"), Ok(Fix::from_num(1.75)));
+"#,
+            if_signed_else_empty_str! {
+                $Signedness;
+                r#"assert_eq!(Fix::from_str("-1.75"), Ok(Fix::from_num(-1.75)));
+"#,
+            },
+            r#"assert_eq!(Fix::from_str("0.00625E+3"), Ok(Fix::from_num(6.25)));
+assert_eq!(Fix::from_str("1.25e-1"), Ok(Fix::from_num(0.125)));
+```
+"#;
+            #[inline]
+            pub const fn from_str(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::from_str_radix(src, 10, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
+            }
+        }
+
         comment! {
             "Parses a string slice containing binary digits to return a fixed-point number.
 
 Rounding is to the nearest, with ties rounded to even.
 
+The number can have an optional exponent. The separator “`e`”, “`E`” or “`@`”
+can be used to start an exponent, which is then followed by an optional sign
+“`+`” or “`-`”, and then by a decimal integer which is the exponent. The parsed
+value is scaled by the radix (2 for binary) to the power of the exponent.
+
+Base-2 exponents are supported too, using the separator “`p`” or “`P`”. The
+parsed value is scaled by 2 to the power of the exponent. For binary, since the
+radix is 2, base-2 exponents are equivalent to the other form of exponent.
+
 # Examples
 
 ```rust
 use fixed::{types::extra::U4, ", $s_fixed, "};
 type Fix = ", $s_fixed, r#"<U4>;
-// 1.75 is 1.11 in binary
-let f = Fix::from_str_binary("1.11");
-let check = Fix::from_bits(0b111 << (4 - 2));
-assert_eq!(f, Ok(check));
+// 1.11 in binary is 1.75
+assert_eq!(Fix::from_str_binary("1.11"), Ok(Fix::from_num(1.75)));
 "#,
             if_signed_else_empty_str! {
                 $Signedness;
-                r#"let neg = Fix::from_str_binary("-1.11");
-assert_eq!(neg, Ok(-check));
+                r#"assert_eq!(Fix::from_str_binary("-1.11"), Ok(Fix::from_num(-1.75)));
 "#,
             },
-            "```
-";
+            r#"
+// 111.0101 in binary is 7.3125
+assert_eq!(Fix::from_str_binary("1.110101e2"), Ok(Fix::from_num(7.3125)));
+assert_eq!(Fix::from_str_binary("11101.01e-2"), Ok(Fix::from_num(7.3125)));
+```
+"#;
             #[inline]
-            pub fn from_str_binary(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::from_str_radix(src, 2)
+            pub const fn from_str_binary(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::from_str_radix(src, 2, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -857,6 +1121,15 @@ assert_eq!(neg, Ok(-check));
 
 Rounding is to the nearest, with ties rounded to even.
 
+The number can have an optional exponent. The separator “`e`”, “`E`” or “`@`”
+can be used to start an exponent, which is then followed by an optional sign
+“`+`” or “`-`”, and then by a decimal integer which is the exponent. The parsed
+value is scaled by 8 to the power of the exponent.
+
+Base-2 exponents are supported too, using the separator “`p`” or “`P`”. The
+parsed value is scaled by 2 to the power of the exponent. For example, for octal
+“`P6`” means ×2⁶, and is equivalent to “`E2`” which means ×8².
+
 # Examples
 
 ```rust
@@ -873,11 +1146,15 @@ assert_eq!(f, Ok(check));
 assert_eq!(neg, Ok(-check));
 "#,
             },
-            "```
-";
+            r#"assert_eq!(Fix::from_str_octal("160e-2"), Ok(check));
+```
+"#;
             #[inline]
-            pub fn from_str_octal(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::from_str_radix(src, 8)
+            pub const fn from_str_octal(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::from_str_radix(src, 8, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -886,27 +1163,39 @@ assert_eq!(neg, Ok(-check));
 
 Rounding is to the nearest, with ties rounded to even.
 
+The number can have an optional exponent. Since “`e`” and “`E`” are valid
+hexadecimal digits, they cannot be used as a separator to start an exponent, so
+“`@`” is used instead. This is then followed by an optional sign “`+`” or “`-`”,
+and then by a decimal integer which is the exponent. The parsed value is scaled
+by 16 to the power of the exponent.
+
+Base-2 exponents are supported too, using the separator “`p`” or “`P`”. The
+parsed value is scaled by 2 to the power of the exponent. For example, for
+hexadecimal “`P8`” means ×2⁸, and is equivalent to “`E2`” which means ×16².
+
 # Examples
 
 ```rust
 use fixed::{types::extra::U4, ", $s_fixed, "};
 type Fix = ", $s_fixed, r#"<U4>;
-// 1.75 is 1.11 in binary, 1.C in hexadecimal
-let f = Fix::from_str_hex("1.C");
-let check = Fix::from_bits(0b111 << (4 - 2));
-assert_eq!(f, Ok(check));
+// 1.C in hexadecimal is 1.75
+assert_eq!(Fix::from_str_hex("1.C"), Ok(Fix::from_num(1.75)));
 "#,
             if_signed_else_empty_str! {
                 $Signedness;
-                r#"let neg = Fix::from_str_hex("-1.C");
-assert_eq!(neg, Ok(-check));
+                r#"assert_eq!(Fix::from_str_hex("-1.C"), Ok(Fix::from_num(-1.75)));
 "#,
             },
-            "```
-";
+            r#"assert_eq!(Fix::from_str_hex("1C@-1"), Ok(Fix::from_num(1.75)));
+assert_eq!(Fix::from_str_hex(".01C@+2"), Ok(Fix::from_num(1.75)));
+```
+"#;
             #[inline]
-            pub fn from_str_hex(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::from_str_radix(src, 16)
+            pub const fn from_str_hex(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::from_str_radix(src, 16, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -934,8 +1223,11 @@ assert_eq!(U8F8::saturating_from_str("-1"), Ok(U8F8::ZERO));
             "```
 ";
             #[inline]
-            pub fn saturating_from_str(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::saturating_from_str_radix(src, 10)
+            pub const fn saturating_from_str(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::saturating_from_str_radix(src, 10, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -963,8 +1255,13 @@ assert_eq!(U8F8::saturating_from_str_binary("-1"), Ok(U8F8::ZERO));
             "```
 ";
             #[inline]
-            pub fn saturating_from_str_binary(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::saturating_from_str_radix(src, 2)
+            pub const fn saturating_from_str_binary(
+                src: &str,
+            ) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::saturating_from_str_radix(src, 2, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -992,8 +1289,13 @@ assert_eq!(U8F8::saturating_from_str_octal("-1"), Ok(U8F8::ZERO));
             "```
 ";
             #[inline]
-            pub fn saturating_from_str_octal(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::saturating_from_str_radix(src, 8)
+            pub const fn saturating_from_str_octal(
+                src: &str,
+            ) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::saturating_from_str_radix(src, 8, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -1021,8 +1323,13 @@ assert_eq!(U8F8::saturating_from_str_hex("-1"), Ok(U8F8::ZERO));
             "```
 ";
             #[inline]
-            pub fn saturating_from_str_hex(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::saturating_from_str_radix(src, 16)
+            pub const fn saturating_from_str_hex(
+                src: &str,
+            ) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::saturating_from_str_radix(src, 16, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -1052,8 +1359,11 @@ assert_eq!(U8F8::wrapping_from_str("-9999.5"), Ok(U8F8::from_num(240.5)));
             "```
 ";
             #[inline]
-            pub fn wrapping_from_str(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::wrapping_from_str_radix(src, 10)
+            pub const fn wrapping_from_str(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::wrapping_from_str_radix(src, 10, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -1083,8 +1393,13 @@ assert_eq!(U8F8::wrapping_from_str_binary("-101100111000.1"), Ok(check.wrapping_
             "```
 ";
             #[inline]
-            pub fn wrapping_from_str_binary(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::wrapping_from_str_radix(src, 2)
+            pub const fn wrapping_from_str_binary(
+                src: &str,
+            ) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::wrapping_from_str_radix(src, 2, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -1114,8 +1429,13 @@ assert_eq!(U8F8::wrapping_from_str_octal("-7165.4"), Ok(check.wrapping_neg()));
             "```
 ";
             #[inline]
-            pub fn wrapping_from_str_octal(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::wrapping_from_str_radix(src, 8)
+            pub const fn wrapping_from_str_octal(
+                src: &str,
+            ) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::wrapping_from_str_radix(src, 8, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -1145,8 +1465,167 @@ assert_eq!(U8F8::wrapping_from_str_hex("-C0F.FE"), Ok(check.wrapping_neg()));
             "```
 ";
             #[inline]
-            pub fn wrapping_from_str_hex(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
-                FromStrRadix::wrapping_from_str_radix(src, 16)
+            pub const fn wrapping_from_str_hex(src: &str) -> Result<$Fixed<Frac>, ParseFixedError> {
+                match from_str::$Inner::wrapping_from_str_radix(src, 16, Self::FRAC_NBITS) {
+                    Ok(bits) => Ok($Fixed::from_bits(bits)),
+                    Err(e) => Err(e),
+                }
+            }
+        }
+
+        comment! {
+            "Parses a string slice containing decimal digits to return a fixed-point number,
+panicking on overflow.
+
+Rounding is to the nearest, with ties rounded to even.
+
+# Panics
+
+Panics if the value does not fit or if there is a parsing error.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+// 1.75 is 1.11 in binary
+let f = Fix::unwrapped_from_str("1.75");
+assert_eq!(f, Fix::from_bits(0b111 << (4 - 2)));
+```
+
+The following panics because of a parsing error.
+
+```rust,should_panic
+use fixed::{types::extra::U4, "#, $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+let _error = Fix::unwrapped_from_str("1.75.");
+```
+"#;
+            #[inline]
+            #[track_caller]
+            #[must_use]
+            pub const fn unwrapped_from_str(src: &str) -> $Fixed<Frac> {
+                match $Fixed::from_str(src) {
+                    Ok(o) => o,
+                    Err(e) => panic!("{}", e.message()),
+                }
+            }
+        }
+
+        comment! {
+            "Parses a string slice containing binary digits to return a fixed-point number,
+panicking on overflow.
+
+Rounding is to the nearest, with ties rounded to even.
+
+# Panics
+
+Panics if the value does not fit or if there is a parsing error.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+// 1.75 is 1.11 in binary
+let f = Fix::unwrapped_from_str_binary("1.11");
+assert_eq!(f, Fix::from_bits(0b111 << (4 - 2)));
+```
+
+The following panics because of a parsing error.
+
+```rust,should_panic
+use fixed::{types::extra::U4, "#, $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+let _error = Fix::unwrapped_from_str_binary("1.2");
+```
+"#;
+            #[inline]
+            #[track_caller]
+            #[must_use]
+            pub const fn unwrapped_from_str_binary(src: &str) -> $Fixed<Frac> {
+                match $Fixed::from_str_binary(src) {
+                    Ok(o) => o,
+                    Err(e) => panic!("{}", e.message()),
+                }
+            }
+        }
+
+        comment! {
+            "Parses a string slice containing octal digits to return a fixed-point number,
+panicking on overflow.
+
+Rounding is to the nearest, with ties rounded to even.
+
+# Panics
+
+Panics if the value does not fit or if there is a parsing error.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+// 1.75 is 1.11 in binary, 1.6 in octal
+let f = Fix::unwrapped_from_str_octal("1.6");
+assert_eq!(f, Fix::from_bits(0b111 << (4 - 2)));
+```
+
+The following panics because of a parsing error.
+
+```rust,should_panic
+use fixed::{types::extra::U4, "#, $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+let _error = Fix::unwrapped_from_str_octal("1.8");
+```
+"#;
+            #[inline]
+            #[track_caller]
+            #[must_use]
+            pub const fn unwrapped_from_str_octal(src: &str) -> $Fixed<Frac> {
+                match $Fixed::from_str_octal(src) {
+                    Ok(o) => o,
+                    Err(e) => panic!("{}", e.message()),
+                }
+            }
+        }
+
+        comment! {
+            "Parses a string slice containing hexadecimal digits to return a fixed-point number,
+wrapping on overflow.
+
+Rounding is to the nearest, with ties rounded to even.
+
+# Panics
+
+Panics if the value does not fit or if there is a parsing error.
+
+# Examples
+
+```rust
+use fixed::{types::extra::U4, ", $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+// 1.75 is 1.11 in binary, 1.C in hexadecimal
+let f = Fix::unwrapped_from_str_hex("1.C");
+assert_eq!(f, Fix::from_bits(0b111 << (4 - 2)));
+```
+
+The following panics because of a parsing error.
+
+```rust,should_panic
+use fixed::{types::extra::U4, "#, $s_fixed, "};
+type Fix = ", $s_fixed, r#"<U4>;
+let _error = Fix::unwrapped_from_str_hex("1.G");
+```
+"#;
+            #[inline]
+            #[track_caller]
+            #[must_use]
+            pub const fn unwrapped_from_str_hex(src: &str) -> $Fixed<Frac> {
+                match $Fixed::from_str_hex(src) {
+                    Ok(o) => o,
+                    Err(e) => panic!("{}", e.message()),
+                }
             }
         }
 
@@ -1179,10 +1658,13 @@ assert_eq!(U8F8::overflowing_from_str("9999.5"), Ok((U8F8::from_num(15.5), true)
             "```
 ";
             #[inline]
-            pub fn overflowing_from_str(
+            pub const fn overflowing_from_str(
                 src: &str,
             ) -> Result<($Fixed<Frac>, bool), ParseFixedError> {
-                FromStrRadix::overflowing_from_str_radix(src, 10)
+                match from_str::$Inner::overflowing_from_str_radix(src, 10, Self::FRAC_NBITS) {
+                    Ok((bits, overflow)) => Ok(($Fixed::from_bits(bits), overflow)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -1215,10 +1697,13 @@ assert_eq!(U8F8::overflowing_from_str_binary("101100111000.1"), Ok((check, true)
             "```
 ";
             #[inline]
-            pub fn overflowing_from_str_binary(
+            pub const fn overflowing_from_str_binary(
                 src: &str,
             ) -> Result<($Fixed<Frac>, bool), ParseFixedError> {
-                FromStrRadix::overflowing_from_str_radix(src, 2)
+                match from_str::$Inner::overflowing_from_str_radix(src, 2, Self::FRAC_NBITS) {
+                    Ok((bits, overflow)) => Ok(($Fixed::from_bits(bits), overflow)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -1251,10 +1736,13 @@ assert_eq!(U8F8::overflowing_from_str_octal("7165.4"), Ok((check, true)));
             "```
 ";
             #[inline]
-            pub fn overflowing_from_str_octal(
+            pub const fn overflowing_from_str_octal(
                 src: &str,
             ) -> Result<($Fixed<Frac>, bool), ParseFixedError> {
-                FromStrRadix::overflowing_from_str_radix(src, 8)
+                match from_str::$Inner::overflowing_from_str_radix(src, 8, Self::FRAC_NBITS) {
+                    Ok((bits, overflow)) => Ok(($Fixed::from_bits(bits), overflow)),
+                    Err(e) => Err(e),
+                }
             }
         }
 
@@ -1287,10 +1775,13 @@ assert_eq!(U8F8::overflowing_from_str_hex("C0F.FE"), Ok((check, true)));
             "```
 ";
             #[inline]
-            pub fn overflowing_from_str_hex(
+            pub const fn overflowing_from_str_hex(
                 src: &str,
             ) -> Result<($Fixed<Frac>, bool), ParseFixedError> {
-                FromStrRadix::overflowing_from_str_radix(src, 16)
+                match from_str::$Inner::overflowing_from_str_radix(src, 16, Self::FRAC_NBITS) {
+                    Ok((bits, overflow)) => Ok(($Fixed::from_bits(bits), overflow)),
+                    Err(e) => Err(e),
+                }
             }
         }
     };
