@@ -1,4 +1,4 @@
-// Copyright © 2018–2022 Trevor Spiteri
+// Copyright © 2018–2023 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -60,7 +60,7 @@ assert_eq!((-two_and_quarter).int(), -three);
             };
             #[inline]
             #[must_use]
-            pub fn int(self) -> $Fixed<Frac> {
+            pub const fn int(self) -> $Fixed<Frac> {
                 Self::from_bits(self.to_bits() & Self::INT_MASK)
             }
         }
@@ -102,7 +102,7 @@ assert_eq!((-two_and_quarter).frac(), three_quarters);
 ";
             #[inline]
             #[must_use]
-            pub fn frac(self) -> $Fixed<Frac> {
+            pub const fn frac(self) -> $Fixed<Frac> {
                 Self::from_bits(self.to_bits() & Self::FRAC_MASK)
             }
         }
@@ -146,17 +146,17 @@ assert_eq!(Fix::from_num(-2.9).round_to_zero(), Fix::from_num(-2));
             };
             #[inline]
             #[must_use]
-            pub fn round_to_zero(self) -> $Fixed<Frac> {
+            pub const fn round_to_zero(self) -> $Fixed<Frac> {
                 if_signed! {
                     $Signedness;
-                    if self.is_negative() && self.frac() != 0 {
-                        let int = self.int();
-                        let increment = Self::from_bits(Self::INT_LSB);
+                    if self.is_negative() && self.frac().to_bits() != 0 {
+                        let int_bits = self.int().to_bits();
+                        let increment = Self::INT_LSB;
                         if Self::INT_NBITS == 1 {
                             // increment is -1, so subtract it
-                            return int - increment;
+                            return Self::from_bits(int_bits - increment);
                         }
-                        return int + increment;
+                        return Self::from_bits(int_bits + increment);
                     }
                 }
                 self.int()
@@ -190,8 +190,9 @@ assert_eq!(Fix::from_num(2.5).ceil(), Fix::from_num(3));
 [`wrapping_ceil`]: Self::wrapping_ceil
 ";
             #[inline]
+            #[track_caller]
             #[must_use]
-            pub fn ceil(self) -> $Fixed<Frac> {
+            pub const fn ceil(self) -> $Fixed<Frac> {
                 let (ceil, overflow) = self.overflowing_ceil();
                 debug_assert!(!overflow, "overflow");
                 let _ = overflow;
@@ -233,8 +234,9 @@ assert_eq!(Fix::from_num(2.5).floor(), Fix::from_num(2));
 [`wrapping_floor`]: Self::wrapping_floor
 ";
             #[inline]
+            #[track_caller]
             #[must_use]
-            pub fn floor(self) -> $Fixed<Frac> {
+            pub const fn floor(self) -> $Fixed<Frac> {
                 let (floor, overflow) = self.overflowing_floor();
                 debug_assert!(!overflow, "overflow");
                 let _ = overflow;
@@ -270,8 +272,9 @@ assert_eq!(Fix::from_num(2.5).round(), Fix::from_num(3));
 [`wrapping_round`]: Self::wrapping_round
 ";
             #[inline]
+            #[track_caller]
             #[must_use]
-            pub fn round(self) -> $Fixed<Frac> {
+            pub const fn round(self) -> $Fixed<Frac> {
                 let (round, overflow) = self.overflowing_round();
                 debug_assert!(!overflow, "overflow");
                 let _ = overflow;
@@ -302,8 +305,9 @@ assert_eq!(Fix::from_num(3.5).round_ties_to_even(), Fix::from_num(4));
 [`wrapping_round_ties_to_even`]: Self::wrapping_round_ties_to_even
 ";
             #[inline]
+            #[track_caller]
             #[must_use]
-            pub fn round_ties_to_even(self) -> $Fixed<Frac> {
+            pub const fn round_ties_to_even(self) -> $Fixed<Frac> {
                 let (round, overflow) = self.overflowing_round_ties_to_even();
                 debug_assert!(!overflow, "overflow");
                 let _ = overflow;
@@ -331,14 +335,15 @@ assert_eq!(Fix::from_num(2.5).checked_ceil(), Some(Fix::from_num(3)));
 ```
 ";
             #[inline]
-            pub fn checked_ceil(self) -> Option<$Fixed<Frac>> {
+            #[must_use]
+            pub const fn checked_ceil(self) -> Option<$Fixed<Frac>> {
                 let (ceil, overflow) = self.overflowing_ceil();
                 if overflow { None } else { Some(ceil) }
             }
         }
 
         comment! {
-            "Checked floor. Rounds to the next integer towards &minus;∞.",
+            "Checked floor. Rounds to the next integer towards &minus;∞. ",
             if_signed_unsigned!(
                 $Signedness,
                 "Returns [`None`] on overflow.
@@ -376,7 +381,8 @@ assert!(AllFrac::MIN.checked_floor().is_none());
             "```
 ";
             #[inline]
-            pub fn checked_floor(self) -> Option<$Fixed<Frac>> {
+            #[must_use]
+            pub const fn checked_floor(self) -> Option<$Fixed<Frac>> {
                 let (floor, overflow) = self.overflowing_floor();
                 if overflow { None } else { Some(floor) }
             }
@@ -402,7 +408,8 @@ assert_eq!(Fix::from_num(2.5).checked_round(), Some(Fix::from_num(3)));
 ```
 ";
             #[inline]
-            pub fn checked_round(self) -> Option<$Fixed<Frac>> {
+            #[must_use]
+            pub const fn checked_round(self) -> Option<$Fixed<Frac>> {
                 let (round, overflow) = self.overflowing_round();
                 if overflow { None } else { Some(round) }
             }
@@ -423,7 +430,8 @@ assert!(Fix::MAX.checked_round_ties_to_even().is_none());
 ```
 ";
             #[inline]
-            pub fn checked_round_ties_to_even(self) -> Option<$Fixed<Frac>> {
+            #[must_use]
+            pub const fn checked_round_ties_to_even(self) -> Option<$Fixed<Frac>> {
                 let (round, overflow) = self.overflowing_round_ties_to_even();
                 if overflow { None } else { Some(round) }
             }
@@ -450,7 +458,7 @@ assert_eq!(Fix::from_num(2.5).saturating_ceil(), Fix::from_num(3));
 ";
             #[inline]
             #[must_use]
-            pub fn saturating_ceil(self) -> $Fixed<Frac> {
+            pub const fn saturating_ceil(self) -> $Fixed<Frac> {
                 let (ceil, overflow) = self.overflowing_ceil();
                 if overflow { Self::MAX } else { ceil }
             }
@@ -496,7 +504,7 @@ assert_eq!(AllFrac::MIN.saturating_floor(), AllFrac::MIN);
 ";
             #[inline]
             #[must_use]
-            pub fn saturating_floor(self) -> $Fixed<Frac> {
+            pub const fn saturating_floor(self) -> $Fixed<Frac> {
                 let (floor, overflow) = self.overflowing_floor();
                 if overflow { Self::MIN } else { floor }
             }
@@ -523,7 +531,7 @@ assert_eq!(Fix::from_num(2.5).saturating_round(), Fix::from_num(3));
 ";
             #[inline]
             #[must_use]
-            pub fn saturating_round(self) -> $Fixed<Frac> {
+            pub const fn saturating_round(self) -> $Fixed<Frac> {
                 let saturated = if self.to_bits() > 0 {
                     $Fixed::MAX
                 } else {
@@ -550,7 +558,7 @@ assert_eq!(Fix::MAX.saturating_round_ties_to_even(), Fix::MAX);
 ";
             #[inline]
             #[must_use]
-            pub fn saturating_round_ties_to_even(self) -> $Fixed<Frac> {
+            pub const fn saturating_round_ties_to_even(self) -> $Fixed<Frac> {
                 let saturated = if self.to_bits() > 0 {
                     $Fixed::MAX
                 } else {
@@ -582,7 +590,7 @@ assert_eq!(Fix::from_num(2.5).wrapping_ceil(), Fix::from_num(3));
 ";
             #[inline]
             #[must_use]
-            pub fn wrapping_ceil(self) -> $Fixed<Frac> {
+            pub const fn wrapping_ceil(self) -> $Fixed<Frac> {
                 self.overflowing_ceil().0
             }
         }
@@ -627,7 +635,7 @@ assert_eq!(AllFrac::MIN.wrapping_floor(), AllFrac::ZERO);
 ";
             #[inline]
             #[must_use]
-            pub fn wrapping_floor(self) -> $Fixed<Frac> {
+            pub const fn wrapping_floor(self) -> $Fixed<Frac> {
                 self.overflowing_floor().0
             }
         }
@@ -653,7 +661,7 @@ assert_eq!(Fix::from_num(2.5).wrapping_round(), Fix::from_num(3));
 ";
             #[inline]
             #[must_use]
-            pub fn wrapping_round(self) -> $Fixed<Frac> {
+            pub const fn wrapping_round(self) -> $Fixed<Frac> {
                 self.overflowing_round().0
             }
         }
@@ -674,7 +682,7 @@ assert_eq!(Fix::MAX.wrapping_round_ties_to_even(), Fix::MIN);
 ";
             #[inline]
             #[must_use]
-            pub fn wrapping_round_ties_to_even(self) -> $Fixed<Frac> {
+            pub const fn wrapping_round_ties_to_even(self) -> $Fixed<Frac> {
                 self.overflowing_round_ties_to_even().0
             }
         }
@@ -712,8 +720,11 @@ let _overflow = Fix::MAX.unwrapped_ceil();
             #[inline]
             #[track_caller]
             #[must_use]
-            pub fn unwrapped_ceil(self) -> $Fixed<Frac> {
-                self.checked_ceil().expect("overflow")
+            pub const fn unwrapped_ceil(self) -> $Fixed<Frac> {
+                match self.checked_ceil() {
+                    Some(s) => s,
+                    None => panic!("overflow"),
+                }
             }
         }
 
@@ -757,8 +768,11 @@ let _overflow = AllFrac::MIN.unwrapped_floor();
             #[inline]
             #[track_caller]
             #[must_use]
-            pub fn unwrapped_floor(self) -> $Fixed<Frac> {
-                self.checked_floor().expect("overflow")
+            pub const fn unwrapped_floor(self) -> $Fixed<Frac> {
+                match self.checked_floor() {
+                    Some(s) => s,
+                    None => panic!("overflow"),
+                }
             }
         }
 
@@ -795,8 +809,11 @@ let _overflow = Fix::MAX.unwrapped_round();
             #[inline]
             #[track_caller]
             #[must_use]
-            pub fn unwrapped_round(self) -> $Fixed<Frac> {
-                self.checked_round().expect("overflow")
+            pub const fn unwrapped_round(self) -> $Fixed<Frac> {
+                match self.checked_round() {
+                    Some(s) => s,
+                    None => panic!("overflow"),
+                }
             }
         }
 
@@ -828,8 +845,11 @@ let _overflow = Fix::MAX.unwrapped_round_ties_to_even();
             #[inline]
             #[track_caller]
             #[must_use]
-            pub fn unwrapped_round_ties_to_even(self) -> $Fixed<Frac> {
-                self.checked_round_ties_to_even().expect("overflow")
+            pub const fn unwrapped_round_ties_to_even(self) -> $Fixed<Frac> {
+                match self.checked_round_ties_to_even() {
+                    Some(s) => s,
+                    None => panic!("overflow"),
+                }
             }
         }
 
@@ -856,9 +876,10 @@ assert_eq!(Fix::from_num(2.5).overflowing_ceil(), (Fix::from_num(3), false));
 ```
 ";
             #[inline]
-            pub fn overflowing_ceil(self) -> ($Fixed<Frac>, bool) {
+            #[must_use]
+            pub const fn overflowing_ceil(self) -> ($Fixed<Frac>, bool) {
                 let int = self.int();
-                if self.frac() == 0 {
+                if self.frac().to_bits() == 0 {
                     return (int, false);
                 }
                 if Self::INT_NBITS == 0 {
@@ -918,7 +939,8 @@ assert_eq!(AllFrac::MIN.overflowing_floor(), (AllFrac::ZERO, true));
             "```
 ";
             #[inline]
-            pub fn overflowing_floor(self) -> ($Fixed<Frac>, bool) {
+            #[must_use]
+            pub const fn overflowing_floor(self) -> ($Fixed<Frac>, bool) {
                 let int = self.int();
                 if_signed! {
                     $Signedness;
@@ -954,7 +976,8 @@ assert_eq!(Fix::from_num(2.5).overflowing_round(), (Fix::from_num(3), false));
 ```
 ";
             #[inline]
-            pub fn overflowing_round(self) -> ($Fixed<Frac>, bool) {
+            #[must_use]
+            pub const fn overflowing_round(self) -> ($Fixed<Frac>, bool) {
                 let int = self.int();
                 if (self.to_bits() & Self::FRAC_MSB) == 0 {
                     return (int, false);
@@ -1010,7 +1033,8 @@ assert_eq!(Fix::MAX.overflowing_round_ties_to_even(), (Fix::MIN, true));
 ```
 ";
             #[inline]
-            pub fn overflowing_round_ties_to_even(self) -> ($Fixed<Frac>, bool) {
+            #[must_use]
+            pub const fn overflowing_round_ties_to_even(self) -> ($Fixed<Frac>, bool) {
                 let int = self.int();
                 if (self.to_bits() & Self::FRAC_MSB) == 0 {
                     return (int, false);
