@@ -1,4 +1,4 @@
-// Copyright © 2018–2022 Trevor Spiteri
+// Copyright © 2018–2023 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or modify it under
 // the terms of either
@@ -13,14 +13,16 @@
 // <https://www.apache.org/licenses/LICENSE-2.0> and
 // <https://opensource.org/licenses/MIT>.
 
+#![allow(deprecated)]
+
 use crate::{
     float_helper,
-    helpers::{FloatKind, FromFloatHelper},
+    helpers::{FloatKind, FromFloatHelper, Private},
     int_helper::IntFixed,
     traits::{Fixed, FixedEquiv, FromFixed, ToFixed},
     types::extra::U0,
     F128Bits, FixedI128, FixedI16, FixedI32, FixedI64, FixedI8, FixedU128, FixedU16, FixedU32,
-    FixedU64, FixedU8,
+    FixedU64, FixedU8, F128,
 };
 use bytemuck::TransparentWrapper;
 use half::{bf16, f16};
@@ -38,26 +40,27 @@ impl ToFixed for bool {
     ///
     /// [`wrapping_to_fixed`]: ToFixed::wrapping_to_fixed
     #[inline]
+    #[track_caller]
     fn to_fixed<F: Fixed>(self) -> F {
-        ToFixed::to_fixed(self as u8)
+        ToFixed::to_fixed(u8::from(self))
     }
 
     /// Converts a [`bool`] to a fixed-point number if it fits, otherwise returns [`None`].
     #[inline]
     fn checked_to_fixed<F: Fixed>(self) -> Option<F> {
-        ToFixed::checked_to_fixed(self as u8)
+        ToFixed::checked_to_fixed(u8::from(self))
     }
 
     /// Convert a [`bool`] to a fixed-point number, saturating if it does not fit.
     #[inline]
     fn saturating_to_fixed<F: Fixed>(self) -> F {
-        ToFixed::saturating_to_fixed(self as u8)
+        ToFixed::saturating_to_fixed(u8::from(self))
     }
 
     /// Converts a [`bool`] to a fixed-point number, wrapping if it does not fit.
     #[inline]
     fn wrapping_to_fixed<F: Fixed>(self) -> F {
-        ToFixed::wrapping_to_fixed(self as u8)
+        ToFixed::wrapping_to_fixed(u8::from(self))
     }
 
     /// Converts a [`bool`] to a fixed-point number.
@@ -67,7 +70,7 @@ impl ToFixed for bool {
     /// wrapped value is returned.
     #[inline]
     fn overflowing_to_fixed<F: Fixed>(self) -> (F, bool) {
-        ToFixed::overflowing_to_fixed(self as u8)
+        ToFixed::overflowing_to_fixed(u8::from(self))
     }
 
     /// Converts a [`bool`] to a fixed-point number, panicking if it
@@ -80,7 +83,7 @@ impl ToFixed for bool {
     #[inline]
     #[track_caller]
     fn unwrapped_to_fixed<F: Fixed>(self) -> F {
-        ToFixed::unwrapped_to_fixed(self as u8)
+        ToFixed::unwrapped_to_fixed(u8::from(self))
     }
 }
 
@@ -102,6 +105,7 @@ macro_rules! impl_int {
             ///
             /// [`wrapping_from_fixed`]: FromFixed::wrapping_from_fixed
             #[inline]
+            #[track_caller]
             fn from_fixed<F: Fixed>(src: F) -> Self {
                 IntFixed::<$Int>::int(FromFixed::from_fixed(src))
             }
@@ -152,6 +156,7 @@ macro_rules! impl_int {
             /// Panics if the value
             /// does not fit, even when debug assertions are not enabled.
             #[inline]
+            #[track_caller]
             fn unwrapped_from_fixed<F: Fixed>(src: F) -> Self {
                 IntFixed::<$Int>::int(FromFixed::unwrapped_from_fixed(src))
             }
@@ -171,6 +176,7 @@ macro_rules! impl_int {
             ///
             /// [`wrapping_to_fixed`]: ToFixed::wrapping_to_fixed
             #[inline]
+            #[track_caller]
             fn to_fixed<F: Fixed>(self) -> F {
                 ToFixed::to_fixed(IntFixed(self).fixed())
             }
@@ -210,6 +216,7 @@ macro_rules! impl_int {
             /// Panics if the value does not fit, even when debug
             /// assertions are not enabled.
             #[inline]
+            #[track_caller]
             fn unwrapped_to_fixed<F: Fixed>(self) -> F {
                 ToFixed::unwrapped_to_fixed(IntFixed(self).fixed())
             }
@@ -284,8 +291,9 @@ macro_rules! impl_float {
             ///
             /// [`wrapping_from_fixed`]: FromFixed::wrapping_from_fixed
             #[inline]
+            #[track_caller]
             fn from_fixed<F: Fixed>(src: F) -> Self {
-                let helper = src.private_to_float_helper();
+                let helper = src.to_float_helper(Private);
                 float_helper::$Float::from_to_float_helper(helper, F::FRAC_NBITS, F::INT_NBITS)
             }
 
@@ -338,6 +346,7 @@ macro_rules! impl_float {
             /// Panics if the value does not fit, even when debug
             /// assertions are not enabled.
             #[inline]
+            #[track_caller]
             fn unwrapped_from_fixed<F: Fixed>(src: F) -> Self {
                 FromFixed::from_fixed(src)
             }
@@ -362,6 +371,8 @@ it panics; if wrapping is required use [`wrapping_to_fixed`] instead.
 [finite]: ", $link, "::is_finite
 ";
                 #[inline]
+                #[track_caller]
+                #[allow(clippy::redundant_closure_call)]
                 fn to_fixed<F: Fixed>(self) -> F {
                     let (wrapped, overflow) = ToFixed::overflowing_to_fixed(self);
                     debug_assert!(!overflow, $overflows_fmt, $overflows_filt(self));
@@ -380,7 +391,7 @@ it panics; if wrapping is required use [`wrapping_to_fixed`] instead.
                 match kind {
                     FloatKind::Finite { .. } => {
                         let helper = FromFloatHelper { kind };
-                        match F::private_overflowing_from_float_helper(helper) {
+                        match F::overflowing_from_float_helper(Private, helper) {
                             (_, true) => None,
                             (wrapped, false) => Some(wrapped),
                         }
@@ -402,11 +413,12 @@ Panics if `self` is [NaN].
 [NaN]: ", $link, "::is_nan
 ";
                 #[inline]
+                #[track_caller]
                 fn saturating_to_fixed<F: Fixed>(self) -> F {
                     let kind =
                         float_helper::$Float::to_float_kind(self, F::FRAC_NBITS, F::INT_NBITS);
                     let helper = FromFloatHelper { kind };
-                    F::private_saturating_from_float_helper(helper)
+                    F::saturating_from_float_helper(Private, helper)
                 }
             }
 
@@ -423,6 +435,7 @@ Panics if `self` is not [finite].
 [finite]: ", $link, "::is_finite
 ";
                 #[inline]
+                #[track_caller]
                 fn wrapping_to_fixed<F: Fixed>(self) -> F {
                     let (wrapped, _) = ToFixed::overflowing_to_fixed(self);
                     wrapped
@@ -450,7 +463,7 @@ Panics if `self` is not [finite].
                     let kind =
                         float_helper::$Float::to_float_kind(self, F::FRAC_NBITS, F::INT_NBITS);
                     let helper = FromFloatHelper { kind };
-                    F::private_overflowing_from_float_helper(helper)
+                    F::overflowing_from_float_helper(Private, helper)
                 }
             }
 
@@ -468,6 +481,7 @@ when debug assertions are not enabled.
 [finite]: ", $link, "::is_finite
 ";
                 #[inline]
+                #[track_caller]
                 fn unwrapped_to_fixed<F: Fixed>(self) -> F {
                     match ToFixed::overflowing_to_fixed(self) {
                         (val, false) => val,
@@ -483,4 +497,5 @@ impl_float! { f16, "f16", "{} overflows", |x| x }
 impl_float! { bf16, "bf16", "{} overflows", |x| x }
 impl_float! { f32, "f32", "{} overflows", |x| x }
 impl_float! { f64, "f64", "{} overflows", |x| x }
+impl_float! { F128, "F128", "F128::from_bits(0x{:032X}) overflows", |x: F128| x.to_bits() }
 impl_float! { F128Bits, "f64", "F128Bits({}) overflows", |x: F128Bits| x.0 }
