@@ -1,4 +1,4 @@
-// Copyright © 2018–2022 Trevor Spiteri
+// Copyright © 2018–2023 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -21,7 +21,10 @@ use crate::{
     FixedU8,
 };
 use core::{
-    fmt::{Debug, Display, Formatter, Result as FmtResult},
+    fmt::{
+        Binary, Debug, Display, Formatter, LowerExp, LowerHex, Octal, Result as FmtResult,
+        UpperExp, UpperHex,
+    },
     iter::{Product, Sum},
     mem,
     ops::{
@@ -434,7 +437,7 @@ impl<F: Fixed> Unwrapped<F> {
     ///     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     ///     [`usize`].
     ///   * A floating-point number of type [`f16`], [`bf16`],
-    ///     [`f32`], [`f64`] or [`F128Bits`]. For this conversion, the
+    ///     [`f32`], [`f64`] or [`F128`]. For this conversion, the
     ///     method rounds to the nearest, with ties rounding to even.
     ///   * Any other number `src` for which [`ToFixed`] is
     ///     implemented, in which case this method returns
@@ -474,7 +477,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// let _overflow = Unwrapped::<I4F4>::from_num(src);
     /// ```
     ///
-    /// [`F128Bits`]: crate::F128Bits
+    /// [`F128`]: crate::F128
     /// [`bf16`]: half::bf16
     /// [`f16`]: half::f16
     /// [finite]: f64::is_finite
@@ -496,7 +499,7 @@ impl<F: Fixed> Unwrapped<F> {
     ///     [`usize`]. Any fractional bits are discarded, which rounds
     ///     towards &minus;∞.
     ///   * A floating-point number of type [`f16`], [`bf16`],
-    ///     [`f32`], [`f64`] or [`F128Bits`]. For this conversion, the
+    ///     [`f32`], [`f64`] or [`F128`]. For this conversion, the
     ///     method rounds to the nearest, with ties rounding to even.
     ///   * Any other type `Dst` for which [`FromFixed`] is
     ///     implemented, in which case this method returns
@@ -529,7 +532,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// let _overflow = src.to_num::<I2F6>();
     /// ```
     ///
-    /// [`F128Bits`]: crate::F128Bits
+    /// [`F128`]: crate::F128
     /// [`bf16`]: half::bf16
     /// [`f16`]: half::f16
     #[inline]
@@ -538,14 +541,53 @@ impl<F: Fixed> Unwrapped<F> {
         Dst::unwrapped_from_fixed(self.0)
     }
 
+    /// Parses a string slice containing decimal digits to return a fixed-point number.
+    ///
+    /// Rounding is to the nearest, with ties rounded to even.
+    ///
+    /// See also
+    /// <code>FixedI32::[unwrapped\_from\_str][FixedI32::unwrapped_from_str]</code>
+    /// and
+    /// <code>FixedU32::[unwrapped\_from\_str][FixedU32::unwrapped_from_str]</code>.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the value does not fit or if there is a parsing error.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use fixed::{types::I8F8, Unwrapped};
+    /// // 16 + 3/4 = 16.75
+    /// let check = Unwrapped(I8F8::from_bits((16 << 8) + (3 << 8) / 4));
+    /// assert_eq!(Unwrapped::<I8F8>::from_str_dec("16.75"), check);
+    /// ```
+    ///
+    /// The following panics because of a parsing error.
+    ///
+    /// ```rust,should_panic
+    /// use fixed::{types::I8F8, Unwrapped};
+    /// let _error = Unwrapped::<I8F8>::from_str_dec("1.2.");
+    /// ```
+    #[inline]
+    #[track_caller]
+    #[must_use]
+    pub fn from_str_dec(src: &str) -> Unwrapped<F> {
+        Unwrapped(F::unwrapped_from_str(src))
+    }
+
     /// Parses a string slice containing binary digits to return a fixed-point number.
     ///
     /// Rounding is to the nearest, with ties rounded to even.
     ///
     /// See also
-    /// <code>FixedI32::[from\_str\_binary][FixedI32::from_str_binary]</code>
+    /// <code>FixedI32::[unwrapped\_from\_str\_binary][FixedI32::unwrapped_from_str_binary]</code>
     /// and
-    /// <code>FixedU32::[from\_str\_binary][FixedU32::from_str_binary]</code>.
+    /// <code>FixedU32::[unwrapped\_from\_str\_binary][FixedU32::unwrapped_from_str_binary]</code>.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the value does not fit or if there is a parsing error.
     ///
     /// # Examples
     ///
@@ -554,9 +596,25 @@ impl<F: Fixed> Unwrapped<F> {
     /// let check = Unwrapped(I8F8::from_bits(0b1110001 << (8 - 1)));
     /// assert_eq!(Unwrapped::<I8F8>::from_str_binary("111000.1"), Ok(check));
     /// ```
+    ///
+    /// The following panics because of a parsing error.
+    ///
+    /// ```rust,should_panic
+    /// use fixed::{types::I8F8, Unwrapped};
+    /// let _error = Unwrapped::<I8F8>::from_str_binary("1.2");
+    /// ```
+    ///
+    /// # Compatibility note
+    ///
+    /// This method either returns [`Ok`] or panics, and never returns [`Err`].
+    /// In version 2, this method will not return a [`Result`], but will return
+    /// the fixed-point number directly similarly to [`from_str_dec`].
+    ///
+    /// [`from_str_dec`]: Self::from_str_dec
     #[inline]
+    #[track_caller]
     pub fn from_str_binary(src: &str) -> Result<Unwrapped<F>, ParseFixedError> {
-        F::from_str_binary(src).map(Unwrapped)
+        Ok(Unwrapped(F::unwrapped_from_str_binary(src)))
     }
 
     /// Parses a string slice containing octal digits to return a fixed-point number.
@@ -564,8 +622,13 @@ impl<F: Fixed> Unwrapped<F> {
     /// Rounding is to the nearest, with ties rounded to even.
     ///
     /// See also
-    /// <code>FixedI32::[from\_str\_octal][FixedI32::from_str_octal]</code> and
-    /// <code>FixedU32::[from\_str\_octal][FixedU32::from_str_octal]</code>.
+    /// <code>FixedI32::[unwrapped\_from\_str\_octal][FixedI32::unwrapped_from_str_octal]</code>
+    /// and
+    /// <code>FixedU32::[unwrapped\_from\_str\_octal][FixedU32::unwrapped_from_str_octal]</code>.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the value does not fit or if there is a parsing error.
     ///
     /// # Examples
     ///
@@ -574,17 +637,39 @@ impl<F: Fixed> Unwrapped<F> {
     /// let check = Unwrapped(I8F8::from_bits(0o1654 << (8 - 3)));
     /// assert_eq!(Unwrapped::<I8F8>::from_str_octal("165.4"), Ok(check));
     /// ```
+    ///
+    /// The following panics because of a parsing error.
+    ///
+    /// ```rust,should_panic
+    /// use fixed::{types::I8F8, Unwrapped};
+    /// let _error = Unwrapped::<I8F8>::from_str_octal("1.8");
+    /// ```
+    ///
+    /// # Compatibility note
+    ///
+    /// This method either returns [`Ok`] or panics, and never returns [`Err`].
+    /// In version 2, this method will not return a [`Result`], but will return
+    /// the fixed-point number directly similarly to [`from_str_dec`].
+    ///
+    /// [`from_str_dec`]: Self::from_str_dec
     #[inline]
+    #[track_caller]
     pub fn from_str_octal(src: &str) -> Result<Unwrapped<F>, ParseFixedError> {
-        F::from_str_octal(src).map(Unwrapped)
+        Ok(Unwrapped(F::unwrapped_from_str_octal(src)))
     }
 
     /// Parses a string slice containing hexadecimal digits to return a fixed-point number.
     ///
     /// Rounding is to the nearest, with ties rounded to even.
     ///
-    /// See also <code>FixedI32::[from\_str\_hex][FixedI32::from_str_hex]</code>
-    /// and <code>FixedU32::[from\_str\_hex][FixedU32::from_str_hex]</code>.
+    /// See also
+    /// <code>FixedI32::[unwrapped\_from\_str\_hex][FixedI32::unwrapped_from_str_hex]</code>
+    /// and
+    /// <code>FixedU32::[unwrapped\_from\_str\_hex][FixedU32::unwrapped_from_str_hex]</code>.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the value does not fit or if there is a parsing error.
     ///
     /// # Examples
     ///
@@ -593,9 +678,25 @@ impl<F: Fixed> Unwrapped<F> {
     /// let check = Unwrapped(I8F8::from_bits(0xFFE));
     /// assert_eq!(Unwrapped::<I8F8>::from_str_hex("F.FE"), Ok(check));
     /// ```
+    ///
+    /// The following panics because of a parsing error.
+    ///
+    /// ```rust,should_panic
+    /// use fixed::{types::I8F8, Unwrapped};
+    /// let _error = Unwrapped::<I8F8>::from_str_hex("1.G");
+    /// ```
+    ///
+    /// # Compatibility note
+    ///
+    /// This method either returns [`Ok`] or panics, and never returns [`Err`].
+    /// In version 2, this method will not return a [`Result`], but will return
+    /// the fixed-point number directly similarly to [`from_str_dec`].
+    ///
+    /// [`from_str_dec`]: Self::from_str_dec
     #[inline]
+    #[track_caller]
     pub fn from_str_hex(src: &str) -> Result<Unwrapped<F>, ParseFixedError> {
-        F::from_str_hex(src).map(Unwrapped)
+        Ok(Unwrapped(F::unwrapped_from_str_hex(src)))
     }
 
     /// Returns the integer part.
@@ -924,6 +1025,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// Panics if the fixed-point number is ≤&nbsp;0.
     #[inline]
     #[track_caller]
+    #[doc(alias("ilog2"))]
     pub fn int_log2(self) -> i32 {
         self.0.int_log2()
     }
@@ -938,10 +1040,26 @@ impl<F: Fixed> Unwrapped<F> {
     /// Panics if the fixed-point number is ≤&nbsp;0.
     #[inline]
     #[track_caller]
+    #[doc(alias("ilog10"))]
     pub fn int_log10(self) -> i32 {
         self.0.int_log10()
     }
 
+    /// Integer logarithm to the specified base, rounded down.
+    ///
+    /// See also <code>FixedI32::[int\_log][FixedI32::int_log]</code> and
+    /// <code>FixedU32::[int\_log][FixedU32::int_log]</code>.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the fixed-point number is ≤&nbsp;0 or if the base is <&nbsp;2.
+    #[inline]
+    #[track_caller]
+    #[doc(alias("ilog"))]
+    pub fn int_log(self, base: u32) -> i32 {
+        self.0.int_log(base)
+    }
+
     /// Reverses the order of the bits of the fixed-point number.
     ///
     /// See also <code>FixedI32::[reverse\_bits][FixedI32::reverse_bits]</code>
@@ -1039,6 +1157,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// let _overflow = Unwr::MIN.dist(Unwr::ZERO);
     /// ```
     #[inline]
+    #[track_caller]
     #[must_use = "this returns the result of the operation, without modifying the original"]
     pub fn dist(self, other: Unwrapped<F>) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_dist(other.0))
@@ -1171,6 +1290,41 @@ impl<F: Fixed> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_mul_add(mul.0, add.0))
     }
 
+    /// Adds `self` to the product `a`&nbsp;×&nbsp;`b`.
+    ///
+    /// See also
+    /// <code>FixedI32::[unwrapped\_add\_prod][FixedI32::unwrapped_add_prod]</code>
+    /// and
+    /// <code>FixedU32::[unwrapped\_add\_prod][FixedU32::unwrapped_add_prod]</code>.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the result does not fit.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use fixed::{types::I16F16, Unwrapped};
+    /// let half = Unwrapped(I16F16::from_num(0.5));
+    /// let three = Unwrapped(I16F16::from_num(3));
+    /// let four = Unwrapped(I16F16::from_num(4));
+    /// assert_eq!(three.add_prod(four, half), Unwrapped(I16F16::from_num(5)));
+    /// ```
+    ///
+    /// The following panics because of overflow.
+    ///
+    /// ```should_panic
+    /// use fixed::{types::I16F16, Unwrapped};
+    /// let max = Unwrapped(I16F16::MAX);
+    /// let _overflow = max.add_prod(max, Unwrapped(I16F16::from_num(3)));
+    /// ```
+    #[inline]
+    #[track_caller]
+    #[must_use]
+    pub fn add_prod(self, a: Unwrapped<F>, b: Unwrapped<F>) -> Unwrapped<F> {
+        Unwrapped(self.0.unwrapped_add_prod(a.0, b.0))
+    }
+
     /// Multiply and accumulate. Adds (`a` × `b`) to `self`.
     ///
     /// See also
@@ -1199,6 +1353,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// acc.mul_acc(Unwrapped(I16F16::MAX), Unwrapped(I16F16::from_num(3)));
     /// ```
     #[inline]
+    #[track_caller]
     pub fn mul_acc(&mut self, a: Unwrapped<F>, b: Unwrapped<F>) {
         self.0.unwrapped_mul_acc(a.0, b.0);
     }
@@ -1358,6 +1513,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// let _overflow = Unwr::from_num(1.5).lerp(Unwr::ZERO, Unwr::MAX);
     /// ```
     #[inline]
+    #[track_caller]
     #[must_use = "this returns the result of the operation, without modifying the original"]
     pub fn lerp(self, start: Unwrapped<F>, end: Unwrapped<F>) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_lerp(start.0, end.0))
@@ -1401,6 +1557,7 @@ impl<F: Fixed> Unwrapped<F> {
     /// let _overflow = Unwr::MAX.inv_lerp(Unwr::ZERO, Unwr::from_num(0.5));
     /// ```
     #[inline]
+    #[track_caller]
     #[must_use]
     pub fn inv_lerp(self, start: Unwrapped<F>, end: Unwrapped<F>) -> Unwrapped<F> {
         Unwrapped(self.0.unwrapped_inv_lerp(start.0, end.0))
@@ -1517,8 +1674,8 @@ impl<F: FixedSigned> Unwrapped<F> {
     ///
     /// ```rust
     /// use fixed::{types::I16F16, Unwrapped};
-    /// assert_eq!(Unwrapped(<I16F16>::from_num(-3.9)).signum(), Unwrapped(I16F16::from_num(-1)));
-    /// assert_eq!(Unwrapped(<I16F16>::ZERO).signum(), Unwrapped(I16F16::from_num(0)));
+    /// assert_eq!(Unwrapped(<I16F16>::from_num(-3.9)).signum(), Unwrapped(I16F16::NEG_ONE));
+    /// assert_eq!(Unwrapped(<I16F16>::ZERO).signum(), Unwrapped(I16F16::ZERO));
     /// assert_eq!(Unwrapped(<I16F16>::from_num(3.9)).signum(), Unwrapped(I16F16::ONE));
     /// ```
     ///
@@ -1800,6 +1957,48 @@ impl<F: Fixed> Debug for Unwrapped<F> {
     }
 }
 
+impl<F: Fixed> Binary for Unwrapped<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        Binary::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> Octal for Unwrapped<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        Octal::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> LowerHex for Unwrapped<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        LowerHex::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> UpperHex for Unwrapped<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        UpperHex::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> LowerExp for Unwrapped<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        LowerExp::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> UpperExp for Unwrapped<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        UpperExp::fmt(&self.0, f)
+    }
+}
+
 impl<F: Fixed> From<F> for Unwrapped<F> {
     /// Wraps a fixed-point number.
     #[inline]
@@ -1813,9 +2012,19 @@ impl<F: Fixed> FromStr for Unwrapped<F> {
     /// Parses a string slice containing decimal digits to return a fixed-point number.
     ///
     /// Rounding is to the nearest, with ties rounded to even.
+    ///
+    /// This method either returns [`Ok`] or panics, and never returns [`Err`].
+    /// The inherent method
+    /// <code>[Unwrapped]&lt;F>::[from\_str\_dec][Unwrapped::from_str_dec]</code>
+    /// returns the value directly instead of a [`Result`].
+    ///
+    /// # Panics
+    ///
+    /// Panics if the value does not fit or if there is a parsing error.
     #[inline]
+    #[track_caller]
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        F::from_str(s).map(Unwrapped)
+        Ok(Unwrapped(F::unwrapped_from_str(s)))
     }
 }
 
@@ -2148,6 +2357,7 @@ impl<F: Fixed> Product<Unwrapped<F>> for Unwrapped<F> {
 }
 
 impl<'a, F: 'a + Fixed> Product<&'a Unwrapped<F>> for Unwrapped<F> {
+    #[track_caller]
     fn product<I>(mut iter: I) -> Unwrapped<F>
     where
         I: Iterator<Item = &'a Unwrapped<F>>,
