@@ -1,4 +1,4 @@
-// Copyright © 2018–2022 Trevor Spiteri
+// Copyright © 2018–2023 Trevor Spiteri
 
 // This library is free software: you can redistribute it and/or
 // modify it under the terms of either
@@ -21,7 +21,10 @@ use crate::{
     FixedU8,
 };
 use core::{
-    fmt::{Debug, Display, Formatter, Result as FmtResult},
+    fmt::{
+        Binary, Debug, Display, Formatter, LowerExp, LowerHex, Octal, Result as FmtResult,
+        UpperExp, UpperHex,
+    },
     iter::{Product, Sum},
     mem,
     ops::{
@@ -431,7 +434,7 @@ impl<F: Fixed> Wrapping<F> {
     ///     [`isize`], [`u8`], [`u16`], [`u32`], [`u64`], [`u128`], or
     ///     [`usize`].
     ///   * A floating-point number of type [`f16`], [`bf16`],
-    ///     [`f32`], [`f64`] or [`F128Bits`]. For this conversion, the
+    ///     [`f32`], [`f64`] or [`F128`]. For this conversion, the
     ///     method rounds to the nearest, with ties rounding to even.
     ///   * Any other number `src` for which [`ToFixed`] is
     ///     implemented, in which case this method returns
@@ -470,11 +473,12 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(dst_float, Wrapping(I4F4::from_bits(0b11100)));
     /// ```
     ///
-    /// [`F128Bits`]: crate::F128Bits
+    /// [`F128`]: crate::F128
     /// [`bf16`]: half::bf16
     /// [`f16`]: half::f16
     /// [finite]: f64::is_finite
     #[inline]
+    #[track_caller]
     pub fn from_num<Src: ToFixed>(src: Src) -> Wrapping<F> {
         Wrapping(src.wrapping_to_fixed())
     }
@@ -491,7 +495,7 @@ impl<F: Fixed> Wrapping<F> {
     ///     [`usize`]. Any fractional bits are discarded, which rounds
     ///     towards &minus;∞.
     ///   * A floating-point number of type [`f16`], [`bf16`],
-    ///     [`f32`], [`f64`] or [`F128Bits`]. For this conversion, the
+    ///     [`f32`], [`f64`] or [`F128`]. For this conversion, the
     ///     method rounds to the nearest, with ties rounding to even.
     ///   * Any other type `Dst` for which [`FromFixed`] is
     ///     implemented, in which case this method returns
@@ -519,7 +523,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(src.to_num::<I2F6>(), wrapped);
     /// ```
     ///
-    /// [`F128Bits`]: crate::F128Bits
+    /// [`F128`]: crate::F128
     /// [`bf16`]: half::bf16
     /// [`f16`]: half::f16
     #[inline]
@@ -874,6 +878,8 @@ impl<F: Fixed> Wrapping<F> {
     ///
     /// Panics if the fixed-point number is ≤&nbsp;0.
     #[inline]
+    #[track_caller]
+    #[doc(alias("ilog2"))]
     pub fn int_log2(self) -> i32 {
         self.0.int_log2()
     }
@@ -887,10 +893,27 @@ impl<F: Fixed> Wrapping<F> {
     ///
     /// Panics if the fixed-point number is ≤&nbsp;0.
     #[inline]
+    #[track_caller]
+    #[doc(alias("ilog10"))]
     pub fn int_log10(self) -> i32 {
         self.0.int_log10()
     }
 
+    /// Integer logarithm to the specified base, rounded down.
+    ///
+    /// See also <code>FixedI32::[int\_log][FixedI32::int_log]</code> and
+    /// <code>FixedU32::[int\_log][FixedU32::int_log]</code>.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the fixed-point number is ≤&nbsp;0 or if the base is <&nbsp;2.
+    #[inline]
+    #[track_caller]
+    #[doc(alias("ilog"))]
+    pub fn int_log(self, base: u32) -> i32 {
+        self.0.int_log(base)
+    }
+
     /// Reverses the order of the bits of the fixed-point number.
     ///
     /// See also <code>FixedI32::[reverse\_bits][FixedI32::reverse_bits]</code>
@@ -1022,6 +1045,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(frac_1_512.recip(), Wrapping(I8F24::ZERO));
     /// ```
     #[inline]
+    #[track_caller]
     #[must_use]
     pub fn recip(self) -> Wrapping<F> {
         Wrapping(self.0.wrapping_recip())
@@ -1052,6 +1076,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(max.next_multiple_of(max_minus_delta), max_minus_delta * 2);
     /// ```
     #[inline]
+    #[track_caller]
     #[must_use]
     pub fn next_multiple_of(self, other: Wrapping<F>) -> Wrapping<F> {
         Wrapping(self.0.wrapping_next_multiple_of(other.0))
@@ -1073,7 +1098,7 @@ impl<F: Fixed> Wrapping<F> {
     /// let four = Wrapping(I16F16::from_num(4));
     /// let max = Wrapping(I16F16::MAX);
     /// assert_eq!(three.mul_add(half, four), Wrapping(I16F16::from_num(5.5)));
-    /// assert_eq!(max.mul_add(three, max), Wrapping(I16F16::from_bits(!0 << 2)));
+    /// assert_eq!(max.mul_add(three, max), max * 4);
     /// ```
     #[inline]
     #[must_use = "this returns the result of the operation, without modifying the original"]
@@ -1081,6 +1106,30 @@ impl<F: Fixed> Wrapping<F> {
         Wrapping(self.0.wrapping_mul_add(mul.0, add.0))
     }
 
+    /// Adds `self` to the product `a`&nbsp;×&nbsp;`b`.
+    ///
+    /// See also
+    /// <code>FixedI32::[wrapping\_add\_prod][FixedI32::wrapping_add_prod]</code>
+    /// and
+    /// <code>FixedU32::[wrapping\_add\_prod][FixedU32::wrapping_add_prod]</code>.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use fixed::{types::I16F16, Wrapping};
+    /// let half = Wrapping(I16F16::from_num(0.5));
+    /// let three = Wrapping(I16F16::from_num(3));
+    /// let four = Wrapping(I16F16::from_num(4));
+    /// let max = Wrapping(I16F16::MAX);
+    /// assert_eq!(four.add_prod(three, half), Wrapping(I16F16::from_num(5.5)));
+    /// assert_eq!(max.add_prod(max, three), max * 4);
+    /// ```
+    #[inline]
+    #[must_use]
+    pub fn add_prod(self, a: Wrapping<F>, b: Wrapping<F>) -> Wrapping<F> {
+        Wrapping(self.0.wrapping_add_prod(a.0, b.0))
+    }
+
     /// Multiply and accumulate. Adds (`a` × `b`) to `self`.
     ///
     /// See also
@@ -1128,6 +1177,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(Wrapping(I16F16::MAX).div_euclid(quarter), check);
     /// ```
     #[inline]
+    #[track_caller]
     #[must_use = "this returns the result of the operation, without modifying the original"]
     pub fn div_euclid(self, divisor: Wrapping<F>) -> Wrapping<F> {
         Wrapping(self.0.wrapping_div_euclid(divisor.0))
@@ -1152,6 +1202,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!((-num).rem_euclid(den), Wrapping(I16F16::from_num(0.5)));
     /// ```
     #[inline]
+    #[track_caller]
     #[must_use = "this returns the result of the operation, without modifying the original"]
     pub fn rem_euclid(self, divisor: Wrapping<F>) -> Wrapping<F> {
         Wrapping(self.0.rem_euclid(divisor.0))
@@ -1178,6 +1229,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!(min.div_euclid_int(-1), min);
     /// ```
     #[inline]
+    #[track_caller]
     #[must_use = "this returns the result of the operation, without modifying the original"]
     pub fn div_euclid_int(self, divisor: F::Bits) -> Wrapping<F> {
         Wrapping(self.0.wrapping_div_euclid_int(divisor))
@@ -1203,6 +1255,7 @@ impl<F: Fixed> Wrapping<F> {
     /// assert_eq!((-num).rem_euclid_int(2), Wrapping(I16F16::from_num(0.5)));
     /// ```
     #[inline]
+    #[track_caller]
     #[must_use = "this returns the result of the operation, without modifying the original"]
     pub fn rem_euclid_int(self, divisor: F::Bits) -> Wrapping<F> {
         Wrapping(self.0.wrapping_rem_euclid_int(divisor))
@@ -1359,11 +1412,11 @@ impl<F: FixedSigned> Wrapping<F> {
     ///     types::{I0F32, I1F31, I16F16},
     ///     Wrapping,
     /// };
-    /// assert_eq!(Wrapping(<I16F16>::from_num(-3.9)).signum(), Wrapping(I16F16::from_num(-1)));
+    /// assert_eq!(Wrapping(<I16F16>::from_num(-3.9)).signum(), Wrapping(I16F16::NEG_ONE));
     /// assert_eq!(Wrapping(<I16F16>::ZERO).signum(), Wrapping(I16F16::ZERO));
     /// assert_eq!(Wrapping(<I16F16>::from_num(3.9)).signum(), Wrapping(I16F16::ONE));
     ///
-    /// assert_eq!(Wrapping(<I1F31>::from_num(0.5)).signum(), Wrapping(I1F31::from_num(-1)));
+    /// assert_eq!(Wrapping(<I1F31>::from_num(0.5)).signum(), Wrapping(I1F31::NEG_ONE));
     /// assert_eq!(Wrapping(<I0F32>::from_num(0.25)).signum(), Wrapping(I0F32::ZERO));
     /// assert_eq!(Wrapping(<I0F32>::from_num(-0.5)).signum(), Wrapping(I0F32::ZERO));
     /// ```
@@ -1589,6 +1642,48 @@ impl<F: Fixed> Debug for Wrapping<F> {
     }
 }
 
+impl<F: Fixed> Binary for Wrapping<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        Binary::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> Octal for Wrapping<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        Octal::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> LowerHex for Wrapping<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        LowerHex::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> UpperHex for Wrapping<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        UpperHex::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> LowerExp for Wrapping<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        LowerExp::fmt(&self.0, f)
+    }
+}
+
+impl<F: Fixed> UpperExp for Wrapping<F> {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        UpperExp::fmt(&self.0, f)
+    }
+}
+
 impl<F: Fixed> From<F> for Wrapping<F> {
     /// Wraps a fixed-point number.
     #[inline]
